// $Id: FcavPacket.v,v 

`include "FcavPacketTimescale.v"

module FcavPacket(
	clock,
	reset_n,
	data_in,
	data_in_valid,	   
	frame_pulse_in,
	data_out,
	data_out_valid,
	frame_pulse_out
	);

	parameter WIDTH = 8;   // bit width
	parameter DEPTH = 16;   // depth of fifo

	output	[(WIDTH-1):0] data_out;
	output  data_out_valid, frame_pulse_out;
	input	clock, reset_n;
   	input 	[(WIDTH-1):0]  data_in;
   	input	data_in_valid, frame_pulse_in;

	reg [(WIDTH-1):0]    fifo [0:(DEPTH-1)];
	reg                  en_fifo[0:(DEPTH-1)];
	reg                  fp_fifo[0:(DEPTH-1)];

   	// write pointer (points to first free slot)
	integer i;    // loop count

	// first stage of fifo is output
	assign data_out = fifo[DEPTH-1];
	assign data_out_valid = en_fifo[DEPTH-1];
	assign frame_pulse_out = fp_fifo[DEPTH-1];

	always @(posedge clock or negedge reset_n)   // trigger new evaluation
		begin
		if (reset_n == 0)   // it's a reset
		begin
			for (i=0; i<DEPTH; i=i+1)   // zero all valid entries
			begin
				fifo[i] 	= 8'b00000000;
				en_fifo[i] 	= 0;			  
				fp_fifo[i]  = 0;
			end
	    end
	    else   // it's a posedge of clock
	    begin
			en_fifo[0] <= data_in_valid;
			fifo[0] <= data_in;	
			fp_fifo[0] <= frame_pulse_in;
			for (i=1; i<DEPTH; i=i+1)   // shift all valid entries
			begin
				fifo[i] <= fifo[i-1];
				en_fifo[i] <= en_fifo[i-1];
				fp_fifo[i] <= fp_fifo[i-1];
			end
		end
	end // always @ (posedge clock or negedge reset_n)

endmodule
