#if !defined(COMMON_DEFS_)
#define COMMON_DEFS_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SpvPlugin.h"

#define DBG_LVL_SPV			0
#define DBG_LVL_TB			1
#define DBG_LVL_USER		2
#define DBG_LVL_WARNING		3

//#define SPV_FATAL(fatalMsg) {SPV_FATAL_MSG(fatalMsg);}
static SPV_FATAL(string fatalMsg){
	SPV_OUT(<<"*** SPV FATAL: " << fatalMsg<<" on Time: "<<SpvSig::time<<endl);
	SpvConfig::StopSim(true);
}

//#define SPV_DEBUG(debugLevel, debugMsg){SPV_DEBUG_MSG(debugMsg,debugLevel);}
static SPV_DEBUG(string debugMsg, int debugLevel = -1){
	int currDebugLevel = 0;
	SD::GetInt(currDebugLevel, "SPV_DEBUG_LEVEL");

	if((debugLevel >= currDebugLevel) || (debugLevel < 0)) {
		SPV_OUT(<<"*** SPV DEBUG: "<<debugMsg<<" on Time: "<<SpvSig::time<<endl);
	}
}

//#define SPV_ERROR(errMsg) {SPV_ERROR_MSG(errMsg);}
static SPV_ERROR(string errMsg){
	bool stopOnError = true;
	SD::GetBool(stopOnError , "SPV_STOP_ON_ERROR");

	SPV_OUT(<<"*** SPV ERROR: "<<errMsg<<" on Time: "<<SpvSig::time<<endl);
	if(stopOnError) 
		SpvConfig::StopSim(true);
}

enum Validity
{
	Valid,
		ValidTruncate,
		MaybeValid,
		Invalid
};

class CommonDefs
{
public:
	enum TimeUnit
	{
		FS,		//femtoseconds
		PS,		//picoseconds
		NS,		//nanoseconds
		US,		//microseconds
		MS,		//milliseconds
		SEC,	//seconds
		DS,		//decaseconds
		KS		//kiloseconds
	};

	static int CommonDefs::GetTimeScale()
	{
		int retVal = FS;
		SD::GetInt(retVal,"N_TIME_SCALE");
		return retVal;
	};

	static void CommonDefs::TimeToSimTime(SpvBitVec& time, TimeUnit timeUnit)
	{
		int i;
		for(i = FS; i < timeUnit; i++)
			time = MultBitVector(time, 1000);
		time = DivBitVector(time, GetTimeScale());
	};

	static void CommonDefs::TimeToSimTime(SPV_UINT64& time, TimeUnit timeUnit)
	{
		int i;
		for(i = FS; i < timeUnit; i++)
			time = time * 1000;
		time = time / GetTimeScale();
	}

	static SpvBitVec MultBitVector(SpvBitVec src, SPV_UINT64 factor)
	{
		SPV_UINT64 tempLow = src(0, 31).Uint();
		SPV_UINT64 tempHigh = src(32, 63).Uint();
		SPV_UINT64 temp = tempLow + (tempHigh << 32);
		temp *= factor;

		SpvBitVec retVal(0, 64);
		retVal(0, 31) = temp;
		retVal(32, 63) = temp >> 32;
		return retVal;
	}

	static SpvBitVec DivBitVector(SpvBitVec src, SPV_UINT64 divisor)
	{
		SPV_UINT64 tempLow = src(0, 31).Uint();
		SPV_UINT64 tempHigh = src(32, 63).Uint();
		SPV_UINT64 temp = tempLow + (tempHigh << 32);

		temp /= divisor;

		SpvBitVec retVal(0, 64);
		retVal(0, 31) = temp;
		retVal(32, 63) = temp >> 32;
		return retVal;
	}

	static SpvBitVec ModBitVector(SpvBitVec src, SPV_UINT64 divisor)
	{
		SPV_UINT64 tempLow = src(0, 31).Uint();
		SPV_UINT64 tempHigh = src(32, 63).Uint();
		SPV_UINT64 temp = tempLow + (tempHigh << 32);

		temp %= divisor;

		SpvBitVec retVal(0, 64);
		retVal(0, 31) = temp;
		retVal(32, 63) = temp >> 32;
		return retVal;
	}
};

#endif