#if !defined(AFX_COMMONINTERFACES_H__8F447F3A_D08B_4194_AE15_0FDCDF869828__INCLUDED_)
#define AFX_COMMONINTERFACES_H__8F447F3A_D08B_4194_AE15_0FDCDF869828__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CommonDefs.h"

struct RegEntry
{
	std::string		m_CommonName;
	unsigned		m_Address;

	RegEntry()
	{
		m_CommonName	= "";
		m_Address		= 0;
	}

	RegEntry(
		std::string		commonName,
		unsigned		address
		)
	{
		m_CommonName	= commonName;
		m_Address		= address;
	}

	RegEntry(const RegEntry& src)
	{
		m_CommonName	= src.m_CommonName;
		m_Address		= src.m_Address;
	}
};

class INamedObject
{
public:
	virtual void SetName(const std::string& name) = 0;
	virtual const std::string& GetName() const = 0;
};

class ISimEventsHandler
{
public:
	virtual void OnSimStop() = 0;

	virtual void OnResetStart() = 0;

	virtual void OnResetEnd() = 0;

	virtual void OnSync() = 0;
};


class IAccessCollector;
class IAccessCollectorHandler
{
public:
	//Fired on latch of read data
	virtual void OnRead(unsigned addr, SpvBitVec data, IAccessCollector* src) = 0;

	//Fired when slave is supposed to latch data
	virtual void OnWrite(unsigned addr, SpvBitVec data, IAccessCollector* src) = 0;
};


class IStreamByteCollectorHandler
{
public:
  //called when collector has captured a data byte
    virtual void OnDataByte(unsigned char byte, unsigned char bitEnable = 0xFF) = 0;
};

class IStreamBitCollectorHandler
{
public:
  //called when collector has captured a data bit
    virtual void OnDataBit(bool bit) = 0;
};

class IStreamByteCollector
{
public:
	virtual void RegisterHandler(IStreamByteCollectorHandler* handler) = 0;
};

class IStreamBitCollector
{
public:
	virtual void RegisterHandler(IStreamBitCollectorHandler* handler) = 0;
};

class IStreamStimulator
{
public:
	virtual void SendData(SpvBitVec data) = 0;
};

class IAccessCollectorHandler2 : public IAccessCollectorHandler
{
public:
	//Fired at start of request
	virtual void OnStartAccess(IAccessCollector* src) = 0;

	//Fired at end of request
	virtual void OnEndAccess(IAccessCollector* src) = 0;
};

class IAccessCollectorHandlerEx : public IAccessCollectorHandler
{
public:
	//Fired when the slave is supposed to
	//to drive its data
	virtual void OnReadStrobe(unsigned addr, IAccessCollector* src) = 0;
};

class IFramer;
class IFrameHandler
{
 public:
   virtual void OnFrame(IFramer *source, const SpvBitVec &  frame, Validity validity = Valid) = 0;
   virtual void Init(IFramer *TX,IFramer *RX)=0;

};

class IFrameHandlerEx : virtual public IFrameHandler
{
 public:
   virtual void OnStartFrame(IFramer *source) = 0;
};


class IAccessCollector
{
public:
	virtual void RegisterHandler(IAccessCollectorHandler* handler) = 0;
};



class IFrameHandler;
class IFramer
{
public:

	virtual void RegisterHandler(IFrameHandler *handler,bool remove_header=true, bool remove_footer=false) = 0;

};

class IAccessStimulator
{
public:
	//Max data size in BYTES
	virtual unsigned GetMaxDataSize() = 0;

	//Min data size in BYTES
	virtual unsigned GetMinDataSize() = 0;

	//data of size greater than that returned by GetMaxDataSize()
	//will cause an exception
	virtual void RequestWrite(unsigned address, SpvBitVec data) = 0;

	//Some implmentations may ignore burst - if GetMaxDataSize() and
	//GetMinDataSize() return the same value, burst is ignored
	virtual void RequestRead(unsigned address, unsigned burst = 4) = 0;
};

class ISequence;

#define DEFAULT_PRIORITY	UINT_MAX / 2
#define INTERRUPT_PRIORITY	UINT_MAX

struct SequenceEntry
{
	enum EntryState
	{
		Complete,
		Pending,
		InProgress,
	} m_EntryState;

	ISequence* m_Sequence;

	bool				m_IsRead;
	SpvBitVec		m_Address;
	SpvBitVec		m_Data;
	SpvBitVec		m_StartTime;
	unsigned			m_Priority;
	SpvBitVec		m_Delay;

	SequenceEntry()
	{
		m_Sequence		= NULL;
		m_EntryState	= Complete;
		m_IsRead		= false;
		m_Address		= 0;
		m_Data			= 0;
		m_StartTime.Resize(64);
		m_StartTime		= SpvSig::time;
		m_Priority		= DEFAULT_PRIORITY;
		m_Delay.Resize(64);
		m_Delay			= 0;
	}

	SequenceEntry& operator  = (const SequenceEntry& src)
	{
		m_Sequence		= src.m_Sequence;
		m_EntryState	= src.m_EntryState;
		m_IsRead		= src.m_IsRead;
		m_Address		= src.m_Address;
		m_Data.Resize(((SequenceEntry&)src).m_Data.Size());
		m_Data			= src.m_Data;
		m_StartTime		= src.m_StartTime;
		m_Priority		= src.m_Priority;
		m_Delay.Copy(((SequenceEntry&)src).m_Delay);

		return *this;
	}

	SequenceEntry(const SequenceEntry& src)
	{
		*this = src;
	}

	virtual void DummyFunc() {}
};

struct AccessEntry : public SequenceEntry
{
	IAccessStimulator*			m_Stm;

	AccessEntry() : SequenceEntry()
	{
		m_Stm = NULL;
	}

	AccessEntry& operator  = (const AccessEntry& src)
	{
		*(SequenceEntry*)this = src;
		m_Stm			= src.m_Stm;

		return *this;
	}

	AccessEntry(const AccessEntry& src)
	{
		*this = src;
	}

	virtual void DummyFunc() {}

};

class ISequenceHandler;
class ISequence
{
public:
	virtual ISequence* NewSequence() = 0;

	virtual void RequestDeleteSequence() = 0;

	virtual void DeleteSequence() = 0;

	virtual void RegisterHandler(ISequenceHandler* handler) = 0;

	virtual AccessEntry* DoSequence(bool isDirectRead, unsigned addr, SpvBitVec data) = 0;

	virtual AccessEntry* StartSequence() = 0;
};

class ISequenceHandler
{
public:
	virtual void OnFinishSequence(ISequence* sequence) = 0;
};

class IComparatorHandler
{
public:
	enum CheckResult
	{
		Good,
		NoMatch,
		Timeout
	};
	enum Certainty
	{
		Certain,
		Uncertain
	};

	virtual void OnCheck(CheckResult result, Certainty certainty) = 0;
};

class IComparator
{
public:
	virtual void RegisterHandler(IComparatorHandler* handler) = 0;
};


class IInvalidStateDetector;

class IInvalidStateHandler
{
public:
	virtual void OnInvalidState(bool asserted, unsigned value, IInvalidStateDetector* src) = 0;
};

class IInternalHandler : public IInvalidStateHandler
{
public:
};


class IInvalidStateDetector
{
public:
	virtual void RegisterHandler(IInvalidStateHandler* handler) = 0;

	virtual void UnregisterHandler(IInvalidStateHandler* handler) = 0;
};

class IInternalCollector : virtual public IInvalidStateDetector
{
public:

};

class IRegFileModel
{
public:
};

#endif // !defined(AFX_COMMONINTERFACES_H__8F447F3A_D08B_4194_AE15_0FDCDF869828__INCLUDED_)
