// SliceCallback.h: interface for the SliceCallback class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SLICECALLBACK_H__A9ED76F7_3201_4F61_B82B_46D58FAB477F__INCLUDED_)
#define AFX_SLICECALLBACK_H__A9ED76F7_3201_4F61_B82B_46D58FAB477F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <SpvSig.h>
#include <crtdbg.h>

struct Bus_Signal{
	char wire_name[200];
	unsigned start;
	unsigned end;
	bool reverse;

	Bus_Signal(){}
	Bus_Signal(const char* wire_name, unsigned start, unsigned end,bool reverse=false)
	{
		strcpy(this->wire_name, wire_name);
		this->start = start;
		this->end = end;
		this->reverse = reverse;
	}

	Bus_Signal(const Bus_Signal& src)
	{
		strcpy(wire_name, src.wire_name);
		start = src.start;
		end  = src.end;
		reverse=src.reverse;
	}
};

class SliceCallback : public SpvVirSig
{
protected:
	SpvSig		*m_Sig;
	unsigned	m_From;
	unsigned	m_To;
    unsigned    m_Value;
	bool m_Reverse;
public:
	SliceCallback();
	SliceCallback(string name, unsigned from, unsigned to, bool reverse, bool checkAlways = false);
	SliceCallback(Bus_Signal busSignal, bool checkAlways = false)
	{
		Init(busSignal, checkAlways);
	}

	virtual ~SliceCallback();

	unsigned GetSize();

	virtual unsigned GetUint(LogicVal lv = bin_Val);

	void Init(Bus_Signal busSignal, bool checkAlways = false)
	{
		Init(busSignal.wire_name, busSignal.start, busSignal.end, busSignal.reverse, checkAlways);
	}

	void Init(string name, unsigned from, unsigned to, bool reverse = false, bool checkAlways = false);

	unsigned eval_Expression_Value(void *user_Data);

	void SetSlice(SpvBitVec  Value, LogicVal lv, bool force = false)
	{
		SetSlice(Value, force, lv);
	}

	void SetSlice(SpvBitVec  Value, bool force = false, LogicVal lv = bin_Val);

	SpvSigSlice& GetRef();

	SpvSig& GetSig();

	bool IsInitialized() { return m_Sig != NULL;};

	SpvAProcCtrl	*create_Process(
		SpvEdge			trigger,
		void			*basic_Derived,
		SpvProcMethod	method_Name,
		unsigned		value	=0,//only for at_Value or at_Not_Value trigger
		spv_Start_Mode	mode	= start_First_Change);

};

class DualSliceCallback : public spv_User_Cb
{
protected:
	SpvSig*		m_Sig1;
	unsigned		m_From1;
	unsigned		m_To1;
	SpvSig*		m_Sig2;
	unsigned		m_From2;
	unsigned		m_To2;

	SpvBitVec	m_Value;

public:
	DualSliceCallback();
	virtual ~DualSliceCallback();


	void Init(
			 string name1,
			 unsigned from1,
			 unsigned to1,
			 string name2,
			 unsigned from2,
			 unsigned to2);

	void Init(Bus_Signal first, Bus_Signal second);

	unsigned eval_Expression_Value(void *user_Data);

	SpvBitVec GetValue();
};
#endif // !defined(AFX_SLICECALLBACK_H__A9ED76F7_3201_4F61_B82B_46D58FAB477F__INCLUDED_)
