// SliceCallback.cpp: implementation of the SliceCallback class.
//
//////////////////////////////////////////////////////////////////////

#include "SliceCallback.h"
#include "MnemonicDefs.h"
#include <crtdbg.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SliceCallback::SliceCallback()
{
	m_Sig = NULL;
}

SliceCallback::SliceCallback(string name, unsigned from, unsigned to,bool reverse, bool checkAlways)
{
	m_Sig = NULL;
	Init(name, from, to, reverse, checkAlways);
}

SliceCallback::~SliceCallback()
{
	delete m_Sig;
}

void SliceCallback::Init(string name, unsigned from, unsigned to,bool reverse, bool checkAlways)
{
	if(name != std::string(SIG_NOT_CONNECTED))
	{
		delete m_Sig;
		m_Sig = new SpvSig(name);
        if(from == UINT_MAX)
            from = m_Sig->size() - 1;
        if(to == UINT_MAX)
            to = m_Sig->size() - 1;

		m_From =	to > from ? from : to;
		m_To =		to < from ? from : to;
		m_Reverse=reverse;
		char fromStr[10], toStr[10];

		itoa(m_From, fromStr, 10);
		itoa(m_To, toStr, 10);

		spv_User_Cb::set_Name(name + "(" + fromStr + "," + toStr + ")");
		//spv_User_Cb::check_At_Every_SimPlus_Tick();


		m_Value = (*m_Sig)(m_From, m_To).uint();


		//multiple wires dont work v3
		if(!checkAlways)
			spv_User_Cb::init(name);
		else
			check_At_Every_SimPlus_Tick();
	}
}

unsigned SliceCallback::eval_Expression_Value(void *user_Data)
{
	_ASSERT(IsInitialized());
	unsigned temp = 0;
	temp = (*m_Sig)(m_From, m_To).uint();
	return temp;
}

SpvAProcCtrl	*SliceCallback::create_Process(
	SPV_PT			trigger,
	void			*basic_Derived,
	SPV_PCBM		method_Name,
	unsigned		value,
	spv_Start_Mode	mode
)
{
	_ASSERT(IsInitialized());
	return spv_User_Cb::create_Process(trigger, basic_Derived, method_Name, value, mode);
}

SpvSigSlice& SliceCallback::GetRef()
{
	return (*m_Sig)(m_From, m_To);
}

SpvSig& SliceCallback::GetSig()
{
	return *m_Sig;
}

unsigned SliceCallback::GetSize()
{
	return  m_To- m_From +1;
}

unsigned  SliceCallback::GetUint(logic_Value lv)
{
	SpvBitVec  value;
//	if (m_From== m_To)
//		Value[0]=(*m_Sig)[m_To](lv);
//	else {
		if (m_Reverse )
		{
			for (int i=0;i<  (m_To - m_From +1);i++)
				value[i]=(*m_Sig)[m_To-i](lv);
		}
		else
			value=(*m_Sig)(m_From, m_To)(lv);
//	}
	return m_Value = value.uint();
}




void  SliceCallback::SetSlice(SpvBitVec  Value, bool force, logic_Value lv)
{
	if (m_Reverse )
	{
		int last_bit=m_To - m_From;
		for (int i=0;i<  (m_To - m_From +1)/2;i++)
		{
			bool tmp=Value[i];
			Value[i]=Value[last_bit-i];
			Value[last_bit-i]=tmp ;
		}
	}

	if(force)
	{
		m_Sig->release_Sig();
		(*m_Sig)(m_From, m_To)(lv) = Value;
		m_Sig->force_Sig();
	}
	else
		(*m_Sig)(m_From, m_To)(lv) = Value;
}


///////////////////////////////////////DualSliceCallback
DualSliceCallback::DualSliceCallback()
{
	m_Sig1 = NULL;
	m_Sig2 = NULL;
}

DualSliceCallback::~DualSliceCallback()
{
	delete m_Sig1;
	delete m_Sig2;
}

void DualSliceCallback::Init(
							 string name1,
							 unsigned from1,
							 unsigned to1,
							 string name2,
							 unsigned from2,
							 unsigned to2
							 )
{
	delete m_Sig1;
	m_Sig1 = new SpvSig(name1);

    m_From1 = from1;
    if(m_From1 == UINT_MAX)
        m_From1 = m_Sig1->size() - 1;

    m_To1 = to1;
    if(m_To1 == UINT_MAX)
        m_To1 = m_Sig1->size() - 1;

    delete m_Sig2;
	m_Sig2 = new SpvSig(name2);

    m_From2 = from2;
    if(m_From2 == UINT_MAX)
        m_From2 = m_Sig2->size() - 1;

    m_To2 = to2;
    if(m_To2 == UINT_MAX)
        m_To2 = m_Sig2->size() - 1;

	char fromStr1[10], toStr1[10];
	char fromStr2[10], toStr2[10];

	itoa(m_From1, fromStr1, 10);
	itoa(m_To1, toStr1, 10);
	itoa(m_From2, fromStr2, 10);
	itoa(m_To2, toStr2, 10);

	spv_User_Cb::set_Name(name1 + "(" + fromStr1 + "," + toStr1 + ") + " +
		name2 + "(" + fromStr2 + "," + toStr2 + ")");

	unsigned width1 = m_To1 - m_From1 + 1;
	unsigned width2 = m_To2 - m_From2 + 1;
	m_Value.resize(width1 + width2);

	m_Value(0, width1 - 1)  = (*m_Sig1)(m_From1, m_To1).uint();
	m_Value(width1, width1 + width2 - 1)  = (*m_Sig2)(m_From2, m_To2).uint();

	std::vector<std::string> sensitive;
	sensitive.push_back(name1);
	sensitive.push_back(name2);
	spv_User_Cb::init(sensitive);
}

void DualSliceCallback::Init(Bus_Signal first, Bus_Signal second)
{
	Init(first.wire_name, first.start, first.end, second.wire_name, second.start, second.end);
}

unsigned DualSliceCallback::eval_Expression_Value(void *user_Data)
{
	return GetValue().uint();
}

SpvBitVec DualSliceCallback::GetValue()
{
	unsigned width1 = m_To1 - m_From1 + 1;
	unsigned width2 = m_To2 - m_From2 + 1;
	m_Value(0, width1 - 1)  = (*m_Sig1)(m_From1, m_To1).uint();
	m_Value(width1, width1 + width2 - 1)  = (*m_Sig2)(m_From2, m_To2).uint();
	return m_Value;
}
