#include "SpvState.h"

SpvTrueStateVal::SpvTrueStateVal()
{
	m_Sig = NULL;
}

SpvTrueStateVal::SpvTrueStateVal(string sigName, SpvCompOp op, unsigned val)
{
	
	Init(sigName,op,val);
}

void SpvTrueStateVal::Init(string sigName, SpvCompOp op, unsigned val)
{
	m_Sig = new SpvSig(sigName);
	AddVal(op,val);
}

void SpvTrueStateVal::AddVal(SpvCompOp op, unsigned val)
{
	ValStateStruct currState;
	currState.m_Val = val;
	currState.m_Op	= op;
	m_ValStates.push_back(currState);
}

bool SpvTrueStateVal::IsTrue()
{
	SPV_DEBUG("SpvTrueStateVal::IsTrue Signal-" + m_Sig->Name() + "="+ Itos(m_Sig->Uint()), DBG_LVL_SPV);
	for(int i = 0; i < m_ValStates.size(); i++){
		switch (m_ValStates[i].m_Op) {
		case Equ :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + "==" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val == m_Sig->Uint())) return false;
			break;
		case NEqu :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + "!=" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val != m_Sig->Uint())) return false;
			break;
		case GT :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + ">" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val > m_Sig->Uint())) return false;
			break;
		case GTEqu :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + ">=" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val >= m_Sig->Uint())) return false;
			break;
		case LT :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + "<" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val < m_Sig->Uint())) return false;
			break;
		case LTEqu :
			SPV_DEBUG("SpvTrueStateVal::IsTrue " + Itos(m_ValStates[i].m_Val) + "<=" + Itos(m_Sig->Uint()), DBG_LVL_SPV);
			if(!(m_ValStates[i].m_Val <= m_Sig->Uint())) return false;
			break;
		default :
			return false;
		}
	}
	SPV_DEBUG("SpvTrueStateVal::IsTrue Returns true", DBG_LVL_SPV);
	return true;
}

SpvTrueStateRange::SpvTrueStateRange()
{
	m_Sig = NULL;
}

SpvTrueStateRange::SpvTrueStateRange(string sigName, SpvRange rangeVals)
{
	Init(sigName,rangeVals);
}

void SpvTrueStateRange::Init(string sigName, SpvRange rangeVals)
{
	m_Sig = new SpvSig(sigName);
	AddRange(rangeVals);
}

void SpvTrueStateRange::Init(string sigName, unsigned from, unsigned to)
{
	SpvRange currRange(from,to);
	Init(sigName, currRange);
}

void SpvTrueStateRange::AddRange(unsigned from, unsigned to)
{
	SpvRange currRange(from,to);
	AddRange(currRange);
}

void SpvTrueStateRange::AddRange(SpvRange rangeVals)
{
	if(rangeVals.from > rangeVals.to) 
		rangeVals.Init(rangeVals.to,rangeVals.from);
	m_Ranges.push_back(rangeVals);
}

bool SpvTrueStateRange::IsTrue()
{
	for(int i = 0; i < m_Ranges.size(); i++){
		if((m_Sig->Uint() >= m_Ranges[i].from) && (m_Sig->Uint() <= m_Ranges[i].to))
			return true;
	}
	return false;
}

SpvState::SpvState()
{
	m_ProcCtrl = NULL;
	m_StartEventExist = false;
	m_StateEval = 0;
	m_StartEventPos = 0;
}

unsigned SpvState::AddStartEvent(SpvEvent *startEvent)
{
	m_StartEventExist = true;

	m_StartEventPos = m_States.size();

	StateStruct currState;
	currState.m_Event		= startEvent;
	currState.m_UseEvent	= true;
	currState.m_UseState	= false;
	currState.m_NextState	= UINT_MAX;
	m_States.push_back(currState);

	return m_StartEventPos;
}

unsigned SpvState::AddEvent(SpvEvent *midEvent)
{
	unsigned pos;
	pos = m_States.size();

	StateStruct currState;
	currState.m_Event		= midEvent;
	currState.m_UseEvent	= true;
	currState.m_UseState	= false;
	currState.m_NextState	= UINT_MAX;
	m_States.push_back(currState);

	return pos;
}

unsigned SpvState::AddState(SpvTrueState *midState)
{
	unsigned pos;
	pos = m_States.size();

	StateStruct currState;
	currState.m_TrueState	= midState;
	currState.m_UseEvent	= false;
	currState.m_UseState	= true;
	currState.m_NextState	= UINT_MAX;
	m_States.push_back(currState);

	return pos;
}

unsigned SpvState::AddState(SpvTrueState *midState, SpvEvent *sampleEvent)
{
	unsigned pos;
	pos = m_States.size();

	StateStruct currState;
	currState.m_TrueState	= midState;
	currState.m_UseEvent	= true;
	currState.m_UseState	= true;
	currState.m_Event		= sampleEvent;
	currState.m_NextState	= UINT_MAX;
	m_States.push_back(currState);

	return pos;
}

unsigned SpvState::AddConnection(unsigned fromId, unsigned toId)
{
	SPV_DEBUG("SpvState::AddConnection - From[" + Itos(fromId) + "] To[" + Itos(toId) + "]", DBG_LVL_SPV);

	if(fromId >= m_States.size())
		SPV_ERROR("SpvState::AddConnection fromId is incorrect - " + Itos(fromId));
	else if(toId >= m_States.size())
		SPV_ERROR("SpvState::AddConnection toId is incorrect - " + Itos(toId));
	else
		m_States[fromId].m_NextState = toId;

	return fromId;
}

void SpvState::Start()
{
	if(!m_StartEventExist) {
		SPV_ERROR("SpvState::Start - Start Event was not declared");
		return;
	}

	if(m_ProcCtrl == NULL) 
		m_ProcCtrl = StartTProc(this, (SPVPM)&SpvState::StateThread);
	else
		SPV_DEBUG("Warning SpvState::Start was already called",DBG_LVL_WARNING);
}

void SpvState::Restart()
{
	if(m_ProcCtrl == NULL) {
		SPV_ERROR("SpvState::Restart - Start function was not called");
		return;
	}

	m_ProcCtrl->Restart();
}

void SpvState::Stop()
{
	if(m_ProcCtrl == NULL) {
		SPV_ERROR("SpvState::Restart - Start function was not called");
		return;
	}

	m_ProcCtrl->Kill();
}

void SpvState::StateThread()
{
	for(;;){
		for(int i = m_StartEventPos; i < m_States.size();)
		{
			if(!m_States[i].m_UseState) {
				SPV_DEBUG("SpvState::StateThread - Waiting for event[" + Itos(i) + "]", DBG_LVL_SPV);
				Wait(*m_States[i].m_Event);
				SPV_DEBUG("SpvState::StateThread - Finished event[" + Itos(i) + "]", DBG_LVL_SPV);
			}
			else
			{
				if(m_States[i].m_UseEvent) 
					do {
						SPV_DEBUG("SpvState::StateThread - Waiting for state event[" + Itos(i) + "]", DBG_LVL_SPV);
						Wait(*m_States[i].m_Event);
						SPV_DEBUG("SpvState::StateThread - Waiting for state event[" + Itos(i) + "]", DBG_LVL_SPV);
					} while(!m_States[i].m_TrueState->IsTrue());
				else 
					if(!m_States[i].m_TrueState->IsTrue()) 
						break;
			}
			i = m_States[i].m_NextState;
			SPV_DEBUG("SpvState::StateThread - Next event[" + Itos(i) + "]", DBG_LVL_SPV);
		}

		SPV_DEBUG("SpvState::StateThread - CheckVal", DBG_LVL_SPV);
		CheckVal();
	}
}
unsigned SpvState::Eval()
{
	SPV_DEBUG("SpvState::Eval", DBG_LVL_SPV);
	return ++m_StateEval;
}
