#if !defined(SPV_STATE_)
#define SPV_STATE_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CommonDefs.h"

class SpvTrueState 
{
protected:
	SpvSig	*m_Sig;
public: 
	virtual bool IsTrue() = 0;
};

class SpvTrueStateRange : public SpvTrueState 
{
private:

	std::vector <SpvRange> m_Ranges;

public:
	SpvTrueStateRange();

	SpvTrueStateRange(string sigName, SpvRange rangeVals);

	void Init(string sigName, SpvRange rangeVals);

	void Init(string sigName, unsigned from, unsigned to);

	void AddRange(SpvRange rangeVals);

	void AddRange(unsigned from, unsigned to);

	virtual bool IsTrue();
};

class SpvTrueStateVal : public SpvTrueState
{
private:
	struct ValStateStruct {
		unsigned	m_Val;
		SpvCompOp	m_Op;
	};

	std::vector <ValStateStruct> m_ValStates;
	
public:
	
	SpvTrueStateVal();

	SpvTrueStateVal(string sigName, SpvCompOp op, unsigned val);

	void Init(string sigName, SpvCompOp op, unsigned val);
	
	void AddVal(SpvCompOp op, unsigned val);

	virtual bool IsTrue();
};

class SpvState : public SpvBase, public SpvVirSig 
{
private:
	unsigned	m_StateEval;

	SpvTProcCtrl *m_ProcCtrl;

	bool		m_StartEventExist;

	unsigned	m_StartEventPos;

	struct StateStruct {
		bool			m_UseEvent;
		bool			m_UseState;
		SpvEvent		*m_Event;
		SpvTrueState	*m_TrueState;
		unsigned		m_NextState;
	};

	std::vector <StateStruct> m_States;

public:

	void		StateThread();

	SpvState();

	unsigned	AddStartEvent(SpvEvent *startEvent);

	unsigned	AddEvent(SpvEvent *midEvent);

	unsigned	AddState(SpvTrueState *midState);

	unsigned	AddState(SpvTrueState *midState, SpvEvent *sampleEvent);

	// Return fromId
	unsigned	AddConnection(unsigned fromId, unsigned toId);

	void		Start();

	void		Restart();

	void		Stop();

	virtual unsigned Eval();

};

#endif

