#if !defined(FCAV_COMMON_)
#define FCAV_COMMON_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SpvPlugin.h"
#include "spvPacketAndTraffic.h"
#include "FcavExtPacketData.h"

class ObjectOffsetField : public SpvField
{
public:
	virtual	SpvField& Generate(SpvUserData *Data=NULL) {
		if(Data) {
			ExtPacketData *packetData = static_cast<ExtPacketData*>(Data->m_AnyPtr);
			int objectNum = Data->m_Value;

			// Offset	0 - 88
			// Size		0 - 16
			// Offset	2/1 - 16+88
			// Size		1	- 0
			// Size		2	- Pix/2

			if(objectNum== 0) 
				packetData->m_ObjectNOffsetGen[objectNum].SetVal(88, true);
			else 
				packetData->m_ObjectNOffsetGen[objectNum].SetVal(
					packetData->m_ObjectNSizeGen[objectNum-1].GetLastVal() +
					packetData->m_ObjectNOffsetGen[objectNum-1].GetLastVal(), true);
			
			MeVec() = packetData->m_ObjectNOffsetGen[objectNum].GetLastVal();
		}
		return *this;
	};
};

class NumObjectsField : public SpvField
{
private:
	ExtPacketData *m_PacketData;

public:
	virtual	SpvField& Generate(SpvUserData *Data=NULL) 
	{	
		if(Data) {
			m_PacketData = static_cast<ExtPacketData*>(Data->m_AnyPtr);

			int simpleHdr = 0;
			int extendHdr= 0;
			SD::GetInt(simpleHdr, "N_FCAV_MODE_SIMPLE_HDR");
			SD::GetInt(extendHdr, "N_FCAV_MODE_EXTEND_HDR");
			if(m_PacketData->m_ModeGen.GetLastVal() == simpleHdr)  
				m_PacketData->m_NumObjGen.SetVal(4,true);
			else 
			{
				if(m_PacketData->m_NumObjGen.GenExist()) 
					m_PacketData->m_NumObjGen.SetGenType(UseGen);
				else
					m_PacketData->m_NumObjGen.SetVal(4,true);
			}

			MeVec() = m_PacketData->m_NumObjGen.Gen();
		}

		return *this;
	};
};

class m_ObjectSizeField : public SpvField
{
	protected:
	int m_ObjectNum;
	ExtPacketData *m_PacketData;

	public:
	virtual	SpvField& Generate(SpvUserData *Data=NULL) 
	{
		if(Data) { 
			m_PacketData = static_cast<ExtPacketData*>(Data->m_AnyPtr);
			m_ObjectNum = Data->m_Value;

			if(m_ObjectNum == 0)
			{
				m_PacketData->m_ObjectNSizeGen[0].SetVal(16 + SD::GetInt("N_FCAV_NUM_PALLETES") * 4, true);
				MeVec() = m_PacketData->m_ObjectNSizeGen[0].GetLastVal();
			}
			else if(m_ObjectNum >= 2) {
				int progressiveFrame	= 0;
				int interlacedFrame		= 1;
				SD::GetInt(progressiveFrame, "N_FCAV_FRAMES_PROGRESSIVE");
				SD::GetInt(interlacedFrame, "N_FCAV_FRAMES_INTERLACED");

				m_PacketData->m_FramesFieldsGen.SetVal(m_PacketData->m_FramesFieldsGen.Gen(), true); // Doesn't change over test
				if(m_PacketData->m_FramesFieldsGen.GetLastVal() == interlacedFrame)
				{
					unsigned oddHeight, evenHeight;
					m_PacketData->m_Video.GetInterlacedOddHeight(oddHeight);
					m_PacketData->m_Video.GetInterlacedEvenHeight(evenHeight);

					if(m_ObjectNum == 2)
						MeVec() = m_PacketData->m_Video.GetWidth() * oddHeight * (m_PacketData->m_Video.GetBpp() / 8);
					else
						MeVec() = m_PacketData->m_Video.GetWidth() * evenHeight * (m_PacketData->m_Video.GetBpp() / 8);
				}
				else
				{
					if(m_ObjectNum == 2)
						MeVec() = m_PacketData->m_Video.GetWidth() * (m_PacketData->m_Video.GetBpp() / 8);
					else
						MeVec() = 0;
				}
				m_PacketData->m_ObjectNSizeGen[m_ObjectNum].SetVal(MeVec().Uint(), true);
			}
		}
		return *this;
	}
};

class VideoField : public SpvField
{
protected:
	int m_ObjectNum;
	ExtPacketData *m_PacketData;

public:
	virtual	SpvField& Generate(SpvUserData *Data=NULL) 
	{
		if(Data) { 
			m_PacketData = static_cast<ExtPacketData*>(Data->m_AnyPtr);
			m_ObjectNum = Data->m_Value;

			if(m_PacketData->m_ObjectNSizeGen[m_ObjectNum].GetLastVal() == 0)  
                SetNotPhysical();
			else  {
				int progressiveFrame	= 0;
				int interlacedFrame		= 1;
				SD::GetInt(progressiveFrame, "N_FCAV_FRAMES_PROGRESSIVE");
				SD::GetInt(interlacedFrame, "N_FCAV_FRAMES_INTERLACED");

				if(m_PacketData->m_FramesFieldsGen.GetLastVal() == interlacedFrame)
				{
					if(m_ObjectNum == 2)
						m_PacketData->m_Video.GetInterlacedOddVector(MeVec());
					else
						m_PacketData->m_Video.GetInterlacedEvenVector(MeVec());
				}
				else // Progressive
				{
					if(m_ObjectNum == 2)
						m_PacketData->m_Video.GetImageVector(MeVec());
					else
						MeVec().Resize(0);
				}
			}
		}
		return *this;
	}

	void SetBitVec(SpvBitVec bitVec) 
	{
		MeVec().Resize(bitVec.Size());
		MeVec().Copy(bitVec);
	};		
};

class AudioField : public SpvField
{
private:
	int m_ObjectNum;
	ExtPacketData *m_PacketData;

public:
	virtual	SpvField& Generate(SpvUserData *Data=NULL) 
	{	
		if(Data) {
			m_PacketData = static_cast<ExtPacketData*>(Data->m_AnyPtr);
			m_ObjectNum = Data->m_Value;

			if(m_PacketData->m_ObjectNSizeGen[m_ObjectNum].GetLastVal() == 0)  
				SetNotPhysical();
			else 
				MeVec().Resize(m_PacketData->m_ObjectNSizeGen[m_ObjectNum].GetLastVal()*8);
		}

		return *this;
	};

};

#endif