#if !defined(FCAV_EXT_PACKET_DATA_)
#define FCAV_EXT_PACKET_DATA_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SpvPlugin.h"
#include "SpvImage.h"
#include "spvPacketAndTraffic.h"

enum FcavGenType {
	UseGen,
	UseRandom,
	UseCnt,
	UseVal
};

class FcavPacketElemGen : public SpvGen
{
private:
	unsigned	m_Cntr;
	FcavGenType m_GenType;
	SpvBitVec	m_LastGen, m_Value;
	SpvGen		*m_Gen;

public:
	FcavPacketElemGen();

	FcavPacketElemGen(const unsigned bitSize, unsigned value);

	FcavPacketElemGen(SpvGen *userGen);

	void		Init(const unsigned bitSize, unsigned value);

	void		Init(SpvGen *userGen);

	unsigned	Gen();

	bool		GenExist() {return (m_Gen != NULL);};

	void		SetGenType(FcavGenType genType) {m_GenType = genType;};

	void		SetVal(unsigned value, bool updateLastVal = false);

	void		SetBitVec(SpvBitVec value, bool updateLastVal = false);

	void		SetGen(SpvGen *userGen);

	unsigned    GetLastVal() {return m_LastGen.Uint();};
};

struct ExtPaletteData
{
	FcavPacketElemGen m_PaletteNumGen;			// 4 Bits

	//! Palette Color
	FcavPacketElemGen m_PaletteColorGen;			// 4 Bits

	//! Red Intensity Value 
	FcavPacketElemGen m_RedIntensityValueGen;	// 4 Bits

	//! Red Intensity Code 
	FcavPacketElemGen m_RedIntensityCodeGen;		// 4 Bits

	//! Green Intensity Value 
	FcavPacketElemGen m_GreenIntensityValueGen;	// 4 Bits

	//! Green Intensity Code 
	FcavPacketElemGen m_GreenIntensityCodeGen;	// 4 Bits

	//! Blue Intensity Value 
	FcavPacketElemGen m_BlueIntensityValueGen;	// 4 Bits

	//! Blue Intensity Code 
	FcavPacketElemGen m_BlueIntensityCodeGen;	// 4 Bits
};

struct ExtPacketData
{
//////////////////////////////////////
//		FC-AV Container Header		//
//////////////////////////////////////

	//! A 32-bit unsigned integer used for identification of a group of Containers.
	FcavPacketElemGen m_ClipIdGen;					// 32 Bits

	//! An 8-bit unsigned integer which represents the encoding rate 
	//! (null, 15, 20, 24, 24*1000/1001, 24 Segmented, 24*1000/1001 Segmented, 25(PAL),
	//! 30, 30*1000/1001(NTSC), 50, 60, 60*1000/1001(NTSC)).
	FcavPacketElemGen m_VideoFrameRateGen;			// 8 Bits

	//! An 8-bit signed integer used for signifying the transmission rate for 
	//! real-time and faster than real-time transmission of video data
	//! (null, 15, 20, 24, 24*1000/1001, 24 Segmented, 24*1000/1001 Segmented, 25(PAL),
	//! 30, 30*1000/1001(NTSC), 50, 60, 60*1000/1001(NTSC)). 
	FcavPacketElemGen m_TransRateGen;				// 8 Bits

	//! An 8-bit unsigned integer which represents the mode 
	//! (simple header or extended header).
	FcavPacketElemGen m_ModeGen;					// 8 Bits

	//! An 8-bit unsigned integer used for signifying the number of Objects in 
	//! the container,  In simple mode the number of objects is 04h.
	FcavPacketElemGen m_NumObjGen;					// 8 Bits

	//! An 8-bit unsigned integer used for signifying the size of the Extended Header.
	FcavPacketElemGen m_SizeExtendHdrGen;			// 8 Bits



//////////////////////////////////////
//		FC-AV Data Header			//
//////////////////////////////////////

	//! Designates the type of data (video - uncompressed, video  compressed, 
	//! Video program, graphics, audio  uncompressed, audio  compressed, ancillary data, 
	//! Full stream structures, or vendor specific) To be contained with in the Object class
	FcavPacketElemGen m_ObjectNTypeGen[4];			// 8 Bits

	//! An unsigned byte used to link an object with another object within the
	//! Same Container.  The value is the Object Number of the associated object.
	FcavPacketElemGen m_ObjectNLinkPtrGen[4];		// 8 Bits

	//! Includes 5 major classifications of types: Null, Assigned, Reserved, Negotiated, 
	//! and Vendor Specific.  We will typically set this to Null except for video objects in which
	//! We will set to D000h.
	FcavPacketElemGen m_ObjectNIndexGen[4];		// 1 Word

	//! Unsigned 32-bit integer depicting the Object Size in bytes. 
	FcavPacketElemGen m_ObjectNSizeGen[4];			// 1 Word
		
	//! Unsigned 32-bit integer which gives the offset in bytes from the beginning of the Container.
	FcavPacketElemGen m_ObjectNOffsetGen[4];		// 1 Word

	//! A 32-bit field which contains a definition for Vendor Specific Classifications.  If used
	//! Bit 31 is set to one, bits 24 through 30 may be defined by the vendor, bits 0 through 23 
	//! Are the IEEE OUI Company ID.
	FcavPacketElemGen m_ObjectNTypeDefGen[4];		// 1 Word
	
//////////////////////////////////////
//		FC-AV Data					//
//////////////////////////////////////
	//! An Unsigned 14-bit integer depicting the total number of rows within the entire
	//! Video frame
	FcavPacketElemGen m_NumRowsGen;					// 14 Bits
	
	//! An Unsigned 14-bit integer depicting the total number of columns within the entire
	//! Video frame.
	FcavPacketElemGen m_NumColumnsGen;				// 14 Bits

	//! A 4-bit field representing the format of the transmitted video. Frames (progressive) and 
	//! Field (interlaced).
	FcavPacketElemGen m_FramesFieldsGen;			// 4 Bits

	//! This 4-bit field represents the format of the pixel data (Monochrome, RGB, RGGB,
	//! RGBA, YIQ, YCbCr, Color Palette; Ancillary Object table, Color Palette; external table).
	FcavPacketElemGen m_ColorInfoTypeGen;		// 4 Bits

	//! This 4-bit field represents the Pixel Aspect ration of width to height of an entire pixel 
	//! (1:1, 1:1.2, or 1.2:1).
	FcavPacketElemGen m_PixelAspectRatioGen;		// 4 Bits

	//! This 4-bit field represents the Pixel Array Order in which video data is packed in the 
	//! Video object with respect to their position in the video frame or field (Left to Right and 
	//! Top to bottom, Bottom to Top and Right to Left, etc.).
	FcavPacketElemGen m_PixelArrayOrderGen;		// 4 Bits

	//! A 32-bit field which contains the packing table formats ( 8, 10, 12, 16, 20, 24, and 32 bit
	//! Samples).
	FcavPacketElemGen m_PackingTableNumGen;		// 4 Bits

	//! These four 4-bit fields specify the number of bits that represent each subpixel within the 
	//! Video object.
	FcavPacketElemGen m_BitsPerSubPixelGen[4];		// 4 Bits

	//! This 8-bit floating point number specifies the overall brightness correction needed to
	//! display an image properly.
	FcavPacketElemGen m_GammaGen;				// 8 Bits

	//! This 4-bit Boolean specifies whether to correct for distortion (no correction, 
	//! tan theta correction, others undefined).
	FcavPacketElemGen m_TanThetaCorrectionGen;	// 4 Bits

	//! This 8-bit integer  is used only for specifying a tile position when image tiling is used.
	FcavPacketElemGen m_TileNumGen;				// 8 Bits

	//! Undefined
	FcavPacketElemGen m_UndefinedGen;			// 12 Bits

	//! This 32-bit unsigned integer specifies the total number of containers per clip
	FcavPacketElemGen m_NumContainerPerClipGen;	// 1 Word

	
	ExtPaletteData m_PaletteData[64];

	SpvImage		m_Video;
};

static void SetDeafultExtPacketData(ExtPacketData *packetData) 
{
	int i;

	for(i = 0; i <4; i++)  {
		if(SD::IsExists("G_FCAV_BITS_PER_SUB_PIXEL", GenType))
			packetData->m_BitsPerSubPixelGen[i].SetGen(SD::GetGen("G_FCAV_BITS_PER_SUB_PIXEL"));
		if(SD::IsExists("G_FCAV_BITS_PER_SUB_PIXEL", GenType, i))
			packetData->m_BitsPerSubPixelGen[i].SetGen(SD::GetGen("G_FCAV_BITS_PER_SUB_PIXEL",i));
	}
	
	if(SD::IsExists("G_FCAV_CLIP_ID",GenType))				packetData->m_ClipIdGen.SetGen(SD::GetGen("G_FCAV_CLIP_ID"));
	if(SD::IsExists("G_FCAV_COLOR_INFO_TYPE",GenType))		packetData->m_ColorInfoTypeGen.SetGen(SD::GetGen("G_FCAV_COLOR_INFO_TYPE"));
	if(SD::IsExists("G_FRAMES_FIELDS",GenType))				packetData->m_FramesFieldsGen.SetGen(SD::GetGen("G_FRAMES_FIELDS"));
	if(SD::IsExists("G_GAMMA",GenType))						packetData->m_GammaGen.SetGen(SD::GetGen("G_GAMMA"));
	if(SD::IsExists("G_MODE",GenType))						packetData->m_ModeGen.SetGen(SD::GetGen("G_MODE"));
	if(SD::IsExists("G_NUM_COLUMNS",GenType))				packetData->m_NumColumnsGen.SetGen(SD::GetGen("G_NUM_COLUMNS"));
	if(SD::IsExists("G_NUM_CONTAINER_PER_CLIP",GenType))	packetData->m_NumContainerPerClipGen.SetGen(SD::GetGen("G_NUM_CONTAINER_PER_CLIP"));
	if(SD::IsExists("G_NUM_OBJ",GenType))					packetData->m_NumObjGen.SetGen(SD::GetGen("G_NUM_OBJ"));
	if(SD::IsExists("G_NUM_ROWS",GenType))					packetData->m_NumRowsGen.SetGen(SD::GetGen("G_NUM_ROWS"));
	for(i = 0; i < 4; i++)
	{
		if(SD::IsExists("G_FCAV_OBJ_INDEX",GenType))		packetData->m_ObjectNIndexGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_INDEX"));
		if(SD::IsExists("G_FCAV_OBJ_INDEX",GenType,i))		packetData->m_ObjectNIndexGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_INDEX",i));
		if(SD::IsExists("G_FCAV_OBJ_LINK_PTR",GenType))		packetData->m_ObjectNLinkPtrGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_LINK_PTR"));
		if(SD::IsExists("G_FCAV_OBJ_LINK_PTR",GenType,i))	packetData->m_ObjectNLinkPtrGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_LINK_PTR",i));
		if(SD::IsExists("G_FCAV_OBJ_SIZE",GenType))			packetData->m_ObjectNSizeGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_SIZE"));
		if(SD::IsExists("G_FCAV_OBJ_SIZE",GenType,i))		packetData->m_ObjectNSizeGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_SIZE",i));
		if(SD::IsExists("G_FCAV_OBJ_TYPE_DEF",GenType))		packetData->m_ObjectNTypeDefGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_TYPE_DEF"));
		if(SD::IsExists("G_FCAV_OBJ_TYPE_DEF",GenType,i))	packetData->m_ObjectNTypeDefGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_TYPE_DEF",i));
		if(SD::IsExists("G_FCAV_OBJ_TYPE",GenType))			packetData->m_ObjectNTypeGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_TYPE"));
		if(SD::IsExists("G_FCAV_OBJ_TYPE",GenType,i))		packetData->m_ObjectNTypeGen[i].SetGen(SD::GetGen("G_FCAV_OBJ_TYPE",i));
	}

	if(SD::IsExists("G_FCAV_PACKING_TABLE_NUM",GenType))	packetData->m_PackingTableNumGen.SetGen(SD::GetGen("G_FCAV_PACKING_TABLE_NUM"));
	if(SD::IsExists("G_FCAV_PIXEL_ARRAY_ORDER",GenType))	packetData->m_PixelArrayOrderGen.SetGen(SD::GetGen("G_FCAV_PIXEL_ARRAY_ORDER"));
	if(SD::IsExists("G_FCAV_PIXEL_ASPECT_RATIO",GenType))	packetData->m_PixelAspectRatioGen.SetGen(SD::GetGen("G_FCAV_PIXEL_ASPECT_RATIO"));
	if(SD::IsExists("G_FCAV_SIZE_EXTEND_HDR",GenType))		packetData->m_SizeExtendHdrGen.SetGen(SD::GetGen("G_FCAV_SIZE_EXTEND_HDR"));
	if(SD::IsExists("G_FCAV_TAN_THETA_CORRECTION",GenType))	packetData->m_TanThetaCorrectionGen.SetGen(SD::GetGen("G_FCAV_TAN_THETA_CORRECTION"));
	if(SD::IsExists("G_FCAV_TILE_NUM",GenType))				packetData->m_TileNumGen.SetGen(SD::GetGen("G_FCAV_TILE_NUM"));
	if(SD::IsExists("G_FCAV_TRANS_RATE",GenType))			packetData->m_TransRateGen.SetGen(SD::GetGen("G_FCAV_TRANS_RATE"));
	if(SD::IsExists("G_FCAV_UNDEFINED",GenType))			packetData->m_UndefinedGen.SetGen(SD::GetGen("G_FCAV_UNDEFINED"));
	if(SD::IsExists("G_FCAV_VIDEO_FRAME_RATE",GenType))		packetData->m_VideoFrameRateGen.SetGen(SD::GetGen("G_FCAV_VIDEO_FRAME_RATE"));

	for(i = 0; i < SD::GetInt("N_FCAV_NUM_PALLETES"); i++)
	{
		if(SD::IsExists("G_FCAV_RED_INTENSITY_CODE",GenType))		packetData->m_PaletteData[i].m_RedIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_RED_INTENSITY_CODE"));
		if(SD::IsExists("G_FCAV_RED_INTENSITY_CODE",GenType,i))		packetData->m_PaletteData[i].m_RedIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_RED_INTENSITY_CODE",i));
		if(SD::IsExists("G_FCAV_RED_INTENSITY_VALUE",GenType))		packetData->m_PaletteData[i].m_RedIntensityValueGen.SetGen(SD::GetGen("G_FCAV_RED_INTENSITY_VALUE"));
		if(SD::IsExists("G_FCAV_RED_INTENSITY_VALUE",GenType,i))	packetData->m_PaletteData[i].m_RedIntensityValueGen.SetGen(SD::GetGen("G_FCAV_RED_INTENSITY_VALUE",i));

		if(SD::IsExists("G_FCAV_GREEN_INTENSITY_CODE",GenType))		packetData->m_PaletteData[i].m_GreenIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_GREEN_INTENSITY_CODE"));
		if(SD::IsExists("G_FCAV_GREEN_INTENSITY_CODE",GenType,i))	packetData->m_PaletteData[i].m_GreenIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_GREEN_INTENSITY_CODE",i));
		if(SD::IsExists("G_FCAV_GREEN_INTENSITY_VALUE",GenType))	packetData->m_PaletteData[i].m_GreenIntensityValueGen.SetGen(SD::GetGen("G_FCAV_GREEN_INTENSITY_VALUE"));
		if(SD::IsExists("G_FCAV_GREEN_INTENSITY_VALUE",GenType,i))	packetData->m_PaletteData[i].m_GreenIntensityValueGen.SetGen(SD::GetGen("G_FCAV_GREEN_INTENSITY_VALUE",i));

		if(SD::IsExists("G_FCAV_BLUE_INTENSITY_CODE",GenType))		packetData->m_PaletteData[i].m_BlueIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_BLUE_INTENSITY_CODE"));
		if(SD::IsExists("G_FCAV_BLUE_INTENSITY_CODE",GenType,i))	packetData->m_PaletteData[i].m_BlueIntensityCodeGen.SetGen(SD::GetGen("G_FCAV_BLUE_INTENSITY_CODE",i));
		if(SD::IsExists("G_FCAV_BLUE_INTENSITY_VALUE",GenType))		packetData->m_PaletteData[i].m_BlueIntensityValueGen.SetGen(SD::GetGen("G_FCAV_BLUE_INTENSITY_VALUE"));
		if(SD::IsExists("G_FCAV_BLUE_INTENSITY_VALUE",GenType,i))	packetData->m_PaletteData[i].m_BlueIntensityValueGen.SetGen(SD::GetGen("G_FCAV_BLUE_INTENSITY_VALUE",i));
		
		if(SD::IsExists("G_FCAV_PALETTE_COLOR",GenType))			packetData->m_PaletteData[i].m_PaletteColorGen.SetGen(SD::GetGen("G_FCAV_PALETTE_COLOR"));
		if(SD::IsExists("G_FCAV_PALETTE_COLOR",GenType,i))			packetData->m_PaletteData[i].m_PaletteColorGen.SetGen(SD::GetGen("G_FCAV_PALETTE_COLOR",i));
		if(SD::IsExists("G_FCAV_PALETTE_NUM",GenType))				packetData->m_PaletteData[i].m_PaletteNumGen.SetGen(SD::GetGen("G_FCAV_PALETTE_NUM"));
		if(SD::IsExists("G_FCAV_PALETTE_NUM",GenType,i))			packetData->m_PaletteData[i].m_PaletteNumGen.SetGen(SD::GetGen("G_FCAV_PALETTE_NUM",i));
	}
};

#endif