#include "FcavPacketContainerHdr.h"

/////////////////////////////////////////////////////////////////////////////
//////			     FC-AV Container Header - Structure				  ///////
/////////////////////////////////////////////////////////////////////////////
FcavPacketTransType::FcavPacketTransType()
{
	m_ReservedA.MakePhysical(this,8);
	m_ReservedB.MakePhysical(this,8);
	m_TransRate.MakePhysical(this,8);
	m_VideoFrameRate.MakePhysical(this,8);
}


FcavPacketContainerType::FcavPacketContainerType()
{
	m_SizeOfExtendedHdr.MakePhysical(this,8);
	m_Reserved.MakePhysical(this,8);
	m_NumberOfObj.MakePhysical(this,8);
	m_Mode.MakePhysical(this,8);
}


FcavPacketContainerHdr::FcavPacketContainerHdr()
{
	m_ContainerCnt.MakePhysical(this,32);
	m_ClipId.MakePhysical(this,32);
	m_ContainerTimeStamp.MakePhysical(this,64);
	m_TransType.MakePhysical(this);
	m_ContainerType.MakePhysical(this);
}

/////////////////////////////////////////////////////////////////////////////
//////			     FC-AV Container Header - Generator				  ///////
/////////////////////////////////////////////////////////////////////////////
FcavPacketTransTypeGenerator::FcavPacketTransTypeGenerator(ExtPacketData *packetData)
{
	m_PacketData = packetData;
	InitKeepConstrains();
}

void FcavPacketTransTypeGenerator::InitKeepConstrains()
{
	// Default Values for Reserved
	int ReservedValA = 0xAA;
	int ReservedValB = 0x55;
	
	SD::GetInt(ReservedValA, "FCAV_TRANS_TYPE_RESERVE_A");
	SD::GetInt(ReservedValB, "FCAV_TRANS_TYPE_RESERVE_B");

	KeepConst(m_ReservedA,ReservedValA,"TransmissionTypeReservedA");
	KeepConst(m_ReservedB,ReservedValB,"TransmissionTypeReservedB");

	KeepGeneration(m_TransRate,&m_PacketData->m_TransRateGen, "TransmissionRate");

	KeepGeneration(m_VideoFrameRate,&m_PacketData->m_VideoFrameRateGen, "VideoFrameRate");
}

FcavPacketContainerTypeGenerator::FcavPacketContainerTypeGenerator(ExtPacketData *packetData)
{
	m_PacketData = packetData;
	InitKeepConstrains();
}

void FcavPacketContainerTypeGenerator::InitKeepConstrains()
{
	KeepGeneration(m_SizeOfExtendedHdr,&m_PacketData->m_SizeExtendHdrGen, "SizeOfExtendedHeadr");

	// Default Values for Reserved
	int ReservedVal = 0x5A;
	
	SD::GetInt(ReservedVal, "FCAV_CONTAINER_TYPE_RESERVE");

	KeepConst(m_Reserved,ReservedVal,"ContainerTypeReserved");

	m_UsrData.m_AnyPtr = (void*)m_PacketData;

	KeepUserData(m_NumberOfObj,			&m_UsrData,		"NumberOfObjects");

	KeepGeneration(m_Mode,&m_PacketData->m_ModeGen, "Mode");
}

FcavPacketContainerHdrGenerator::FcavPacketContainerHdrGenerator(ExtPacketData *packetData) :
	m_TransTypeGenerator(packetData),
	m_ContainerTypeGenerator(packetData)
{	
	m_PacketData = packetData;
	InitKeepConstrains();
};


void FcavPacketContainerHdrGenerator::InitKeepConstrains()
{
	SpvRange ContainerRange[] = {0,UINT_MAX};
	KeepGeneration(m_ContainerCnt, new SpvGenInRangeListOrder(32,ContainerRange,1), "ContainerCnt");

	KeepGeneration(m_ClipId,			&m_PacketData->m_ClipIdGen,	"ClipId");
	
	SetFieldName(m_ContainerTimeStamp,								"TimeStamp");

	KeepNestedPacket(m_TransType,		&m_TransTypeGenerator,		"TransType");

	KeepNestedPacket(m_ContainerType,	&m_ContainerTypeGenerator,	"ContainerType");
}
