#if !defined(FCAV_PACKET_CONTAINER_HDR_)
#define FCAV_PACKET_CONTAINER_HDR_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Common/CommonPacketStruct.h"
#include "FcavCommon.h"

/////////////////////////////////////////////////////////////////////////////
//////			     FC-AV Container Header - Structure				  ///////
/////////////////////////////////////////////////////////////////////////////
class FcavPacketTransType : public SpvPacket
{
protected:
	SpvField				m_ReservedA;			// 8 Bit

	SpvField				m_ReservedB;			// 8 Bit

	//! An 8-bit signed integer used for signifying the transmission rate for 
	//! real-time and faster than real-time transmission of video data
	//! (null, 15, 20, 24, 24*1000/1001, 24 Segmented, 24*1000/1001 Segmented, 25(PAL),
	//! 30, 30*1000/1001(NTSC), 50, 60, 60*1000/1001(NTSC)). 
	SpvField				m_TransRate;			// 8 Bit

	//! An 8-bit unsigned integer which represents the encoding rate 
	//! (null, 15, 20, 24, 24*1000/1001, 24 Segmented, 24*1000/1001 Segmented, 25(PAL),
	//! 30, 30*1000/1001(NTSC), 50, 60, 60*1000/1001(NTSC)). 
	SpvField				m_VideoFrameRate;		// 8 Bit

public:
	FcavPacketTransType();							// 1 Word
};

class FcavPacketContainerType : public SpvPacket
{
protected:
	//! An 8-bit unsigned integer used for signifying the size of the Extended Header.
	SpvField				m_SizeOfExtendedHdr;	// 8 Bit

	SpvField				m_Reserved;				// 8 Bit

	//! An 8-bit unsigned integer used for signifying the number of Objects in 
	//! the container,  In simple mode the number of objects is 04h.
	NumObjectsField			m_NumberOfObj;			// 8 Bit

	//! An 8-bit unsigned integer which represents the mode 
	//! (simple header or extended header).
	SpvField				m_Mode;					// 8 Bit

public:
	FcavPacketContainerType();						// 1 Word
};

//! FC-AV Container Header Contained within the Anonymous Subscriber Messages (ASM) Message Payload
class FcavPacketContainerHdr : public SpvPacket
{
protected:
	
	FcavPacketContainerType	m_ContainerType;		// 1 Word

	FcavPacketTransType		m_TransType;			// 1 Word

	//! A 64-bit word used to store a FC-AV time stamp for transmission purposes
	TimeStampField			m_ContainerTimeStamp;	// 2 Words

	//! A 32-bit unsigned integer used for identification of a group of Containers.
	SpvField				m_ClipId;				// 1 Word

	//! A 32-bit unsigned integer used to identify the position of a Container in a series of 
	//! Containers of the same Clip Id.
	SpvField				m_ContainerCnt;			// 1 Word

public:
	FcavPacketContainerHdr();						// 6 Words
};


/////////////////////////////////////////////////////////////////////////////
//////			     FC-AV Container Header - Generator				  ///////
/////////////////////////////////////////////////////////////////////////////
class	FcavPacketTransTypeGenerator : public SpvPacketGen <FcavPacketTransType>
{
protected:
	SpvUserData							m_UsrData;
	ExtPacketData						*m_PacketData;

public:
	FcavPacketTransTypeGenerator(ExtPacketData *packetData);

	virtual void InitKeepConstrains();
};

class	FcavPacketContainerTypeGenerator : public SpvPacketGen <FcavPacketContainerType>
{
protected:
	SpvUserData							m_UsrData;
	ExtPacketData						*m_PacketData;

public:
	FcavPacketContainerTypeGenerator(ExtPacketData *packetData);

	virtual void InitKeepConstrains();
};

class	FcavPacketContainerHdrGenerator : public SpvPacketGen <FcavPacketContainerHdr>
{
protected:
	ExtPacketData						*m_PacketData;
	FcavPacketContainerTypeGenerator	m_ContainerTypeGenerator;		
	FcavPacketTransTypeGenerator		m_TransTypeGenerator;			

public:
	FcavPacketContainerHdrGenerator(ExtPacketData *packetData);

	virtual void InitKeepConstrains();
};

#endif