#include "FcavPacketData.h"

/////////////////////////////////////////////////////////////////////////////
//////			           FC-AV Data - Structure				      ///////
/////////////////////////////////////////////////////////////////////////////
FcavPacketColorPalette::FcavPacketColorPalette()
{
	m_BlueIntensityCode.MakePhysical(this,4);
	m_BlueIntensityValue.MakePhysical(this,4);
	m_GreenIntensityCode.MakePhysical(this,4);
	m_GreenIntensityValue.MakePhysical(this,4);
	m_RedIntensityCode.MakePhysical(this,4);
	m_RedIntensityValue.MakePhysical(this,4);
	m_PaletteColor.MakePhysical(this,4);
	m_PaletteNum.MakePhysical(this,4);
}

FcavPacketAncillaryData::FcavPacketAncillaryData()
{
	int i;

	// Word 0
	m_FramesFields.MakePhysical(this,4);
	m_NumColumns.MakePhysical(this,14);
	m_NumRows.MakePhysical(this,14);

	// Word 1
	for(i = 3; i >= 0; i--)
		m_BitsPerSubPixel[i].MakePhysical(this,4);
	m_PackingTableNum.MakePhysical(this,4);
	m_PixelArrayOrder.MakePhysical(this,4);
	m_PixelAspectRatio.MakePhysical(this,4);
	m_ColorInfoType.MakePhysical(this,4);

	// Word 2
	m_Undefined.MakePhysical(this,12);
	m_TileNum.MakePhysical(this,8);
	m_TanThetaCorrection.MakePhysical(this,4);
	m_Gamma.MakePhysical(this,8);
	
	// Word 3
	m_NumContainerPerClip.MakePhysical(this,32);
	
	for(i = 0; i < SD::GetInt("N_FCAV_NUM_PALLETES"); i++)
		m_ColorPalette[i].MakePhysical(this);
}

FcavPacketAudioData::FcavPacketAudioData()
{
	m_AudioData.MakePhysical(this/*0bit|32bit*/);
}

FcavPacketVideoData::FcavPacketVideoData()
{
	m_VideoData.MakePhysical(this/*0-12MB*/);
}

FcavPacketData::FcavPacketData()
{
	m_AncillaryObj0.MakePhysical(this);	
	m_AudioObj1.MakePhysical(this);
	m_VideoObj2.MakePhysical(this);
	m_VideoObj3.MakePhysical(this);
}

/////////////////////////////////////////////////////////////////////////////
//////			         FC-AV Data - Generator					      ///////
/////////////////////////////////////////////////////////////////////////////
FcavPacketColorPaletteGenerator::FcavPacketColorPaletteGenerator(ExtPacketData *packetData, int colorPaletteNum)
{
	m_ColorPaletteNum = colorPaletteNum;
	m_PacketData = packetData;
	InitKeepConstrains();
}

void FcavPacketColorPaletteGenerator::InitKeepConstrains()
{
	
	KeepGeneration(m_BlueIntensityCode,		&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_BlueIntensityCodeGen,		"BlueIntensityCode"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_BlueIntensityValue,	&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_BlueIntensityValueGen,	"BlueIntensityValue"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_GreenIntensityCode,	&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_GreenIntensityCodeGen,	"GreenIntensityCode"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_GreenIntensityValue,	&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_GreenIntensityValueGen,	"GreenIntensityValue"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_RedIntensityCode,		&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_RedIntensityCodeGen,		"RedIntensityCode"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_RedIntensityValue,		&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_RedIntensityValueGen,		"RedIntensityValue"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_PaletteColor,			&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_PaletteColorGen,			"PaletteColor"+Itos(m_ColorPaletteNum));
	KeepGeneration(m_PaletteNum,			&m_PacketData->m_PaletteData[m_ColorPaletteNum].m_PaletteNumGen,			"PaletteNum"+Itos(m_ColorPaletteNum));
}

FcavPacketAncillaryDataGenerator::FcavPacketAncillaryDataGenerator(ExtPacketData *packetData, int objectNum) 
{
	m_ObjectNum		= objectNum;
	m_PacketData = packetData;

	m_ColorPaletteGenerator = new FcavPacketColorPaletteGenerator*[SD::GetInt("N_FCAV_NUM_PALLETES")];
	for(int i = 0; i < SD::GetInt("N_FCAV_NUM_PALLETES"); i++)
		m_ColorPaletteGenerator[i] = new FcavPacketColorPaletteGenerator(packetData, i);

	
	InitKeepConstrains();
}

void FcavPacketAncillaryDataGenerator::InitKeepConstrains()
{
	int i;

	// Word 0
	KeepGeneration(m_FramesFields,				&m_PacketData->m_FramesFieldsGen,		"FramesFields");
	KeepGeneration(m_NumColumns,				&m_PacketData->m_NumColumnsGen,			"NumColumns");
	KeepGeneration(m_NumRows,					&m_PacketData->m_NumRowsGen,			"NumberRows");
	
	// Word 1
	for(i = 3; i >= 0; i--) 
		KeepGeneration(m_BitsPerSubPixel[i],	&m_PacketData->m_BitsPerSubPixelGen[i],	"BitsPerSubPixel" + Itos(i));
	KeepGeneration(m_PackingTableNum,			&m_PacketData->m_PackingTableNumGen,	"PackingTableNum");
	KeepGeneration(m_PixelArrayOrder,			&m_PacketData->m_PixelArrayOrderGen,	"PixelArrayOrder");
	KeepGeneration(m_PixelAspectRatio,			&m_PacketData->m_PixelAspectRatioGen,	"PixelAspectRatio");
	KeepGeneration(m_ColorInfoType,				&m_PacketData->m_ColorInfoTypeGen,		"ColorInfoType");
	
	// Word 2
	KeepGeneration(m_Undefined,					&m_PacketData->m_UndefinedGen,			"Undefined");
	KeepGeneration(m_TileNum,					&m_PacketData->m_TileNumGen,				"TileNum");
	KeepGeneration(m_TanThetaCorrection,		&m_PacketData->m_TanThetaCorrectionGen,	"TanThetaCorrection");
	KeepGeneration(m_Gamma,						&m_PacketData->m_GammaGen,				"Gamma");

	// Word 3
	KeepGeneration(m_NumContainerPerClip,		&m_PacketData->m_NumContainerPerClipGen,	"NumContainerPerClip");

	for(i = 0; i < SD::GetInt("N_FCAV_NUM_PALLETES"); i++)
		KeepNestedPacket(m_ColorPalette[i],		m_ColorPaletteGenerator[i],		"ColorPalette" + Itos(i));
}

FcavPacketAudioDataGenerator::FcavPacketAudioDataGenerator(ExtPacketData *packetData, int objectNum)
{
	m_ObjectNum		= objectNum;
	m_PacketData	= packetData;
	InitKeepConstrains();
}

void FcavPacketAudioDataGenerator::InitKeepConstrains()
{
	m_UsrData.m_Value = m_ObjectNum;
	m_UsrData.m_AnyPtr = (void*)m_PacketData;

	KeepUserData(m_AudioData,			&m_UsrData,		"AudioData");
}

FcavPacketVideoDataGenerator::FcavPacketVideoDataGenerator(ExtPacketData *packetData, int objectNum)
{
	m_ObjectNum		= objectNum;
	m_PacketData	= packetData;
	InitKeepConstrains();
}

void FcavPacketVideoDataGenerator::InitKeepConstrains()
{
	m_UsrData.m_Value = m_ObjectNum;
	m_UsrData.m_AnyPtr = (void*)m_PacketData;

	KeepUserData(m_VideoData,			&m_UsrData,		"VideoData" + Itos(m_ObjectNum));
}

FcavPacketDataGenerator::FcavPacketDataGenerator(ExtPacketData *packetData) :
	m_AncillaryGeneratorObj0(packetData,0),
	m_AudioGeneratorObj1(packetData,1),
	m_VideoGeneratorObj2(packetData,2),
	m_VideoGeneratorObj3(packetData,3)
{
	m_PacketData = packetData;
	InitKeepConstrains();
}

void FcavPacketDataGenerator::InitKeepConstrains()
{
	KeepNestedPacket(m_AncillaryObj0,	&m_AncillaryGeneratorObj0,	"m_AncillaryObj0");
	KeepNestedPacket(m_AudioObj1,		&m_AudioGeneratorObj1,		"m_AudioObj1");
	KeepNestedPacket(m_VideoObj2,		&m_VideoGeneratorObj2,		"m_VideoObj2");
	KeepNestedPacket(m_VideoObj3,		&m_VideoGeneratorObj3,		"m_VideoObj3");
}
