#if !defined(FCAV_PACKET_DATA_)
#define FCAV_PACKET_DATA_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Common/CommonPacketStruct.h"
#include "FcavCommon.h"

/////////////////////////////////////////////////////////////////////////////
//////			           FC-AV Data - Structure				      ///////
/////////////////////////////////////////////////////////////////////////////
class FcavPacketColorPalette : public SpvPacket
{
protected:
	int						m_ColorPaletteNum;

	//! Blue Intensity Code 
	SpvField				m_BlueIntensityCode;	// 4 Bits

	//! Blue Intensity Value 
	SpvField				m_BlueIntensityValue;	// 4 Bits

	//! Green Intensity Code 
	SpvField				m_GreenIntensityCode;	// 4 Bits

	//! Green Intensity Value 
	SpvField				m_GreenIntensityValue;	// 4 Bits

	//! Red Intensity Code 
	SpvField				m_RedIntensityCode;		// 4 Bits

	//! Red Intensity Value 
	SpvField				m_RedIntensityValue;	// 4 Bits

	//! Palette Color
	SpvField				m_PaletteColor;			// 4 Bits

	//! Palette Number
	SpvField				m_PaletteNum;			// 4 Bits


public:
	FcavPacketColorPalette();
};

class FcavPacketAncillaryData : public SpvPacket
{
protected:
	//! A 4-bit field representing the format of the transmitted video. Frames (progressive) and 
	//! Field (interlaced).
	SpvField				m_FramesFields;			// 4 Bits

	//! An Unsigned 14-bit integer depicting the total number of columns within the entire
	//! Video frame.
	SpvField				m_NumColumns;			// 14 Bits

	//! An Unsigned 14-bit integer depicting the total number of rows within the entire
	//! Video frame.	
	SpvField				m_NumRows;				// 14 Bits

	//! These four 4-bit fields specify the number of bits that represent each subpixel within the 
	//! Video object.
	SpvField				m_BitsPerSubPixel[4];		// 4 Bits

	//! A 32-bit field which contains the packing table formats ( 8, 10, 12, 16, 20, 24, and 32 bit
	//! Samples).
	SpvField				m_PackingTableNum;		// 4 Bits

	//! This 4-bit field represents the Pixel Array Order in which video data is packed in the 
	//! Video object with respect to their position in the video frame or field (Left to Right and 
	//! Top to bottom, Bottom to Top and Right to Left, etc.).
	SpvField				m_PixelArrayOrder;		// 4 Bits

	//! This 4-bit field represents the Pixel Aspect ration of width to height of an entire pixel 
	//! (1:1, 1:1.2, or 1.2:1).
	SpvField				m_PixelAspectRatio;		// 4 Bits

	//! This 4-bit field represents the format of the pixel data (Monochrome, RGB, RGGB,
	//! RGBA, YIQ, YCbCr, Color Palette; Ancillary Object table, Color Palette; external table).
	SpvField				m_ColorInfoType;		// 4 Bits

	//! Undefined
	SpvField				m_Undefined;			// 12 Bits

	//! This 8-bit integer  is used only for specifying a tile position when image tiling is used.
	SpvField				m_TileNum;				// 8 Bits

	//! This 4-bit Boolean specifies whether to correct for distortion (no correction, 
	//! tan theta correction, others undefined).
	SpvField				m_TanThetaCorrection;	// 4 Bits

	//! This 8-bit floating point number specifies the overall brightness correction needed to
	//! display an image properly.
	SpvField				m_Gamma;				// 8 Bits

	//! This 32-bit unsigned integer specifies the total number of containers per clip
	SpvField				m_NumContainerPerClip;	// 1 Word

	FcavPacketColorPalette	m_ColorPalette[64];		// 4 words to 256 Words

public:
	FcavPacketAncillaryData();						// 4 to 259 Words
};

class FcavPacketAudioData :	public SpvPacket
{
protected:	
	//! Audio Data
	AudioField				m_AudioData;			// 0 to 1 Word
public:
	FcavPacketAudioData();							// 0 to 1 Word
};

class FcavPacketVideoData : public SpvPacket
{
protected:
	//! These objects will be comprised of Video data.  The order of the data is 
	//! Governed by the Pixel Array Order field defined in Object 0 word 1.  If the 
	//! Video format is frame based (progressive) only Object 2 will contain data. 
	//! If the Video data is field based (interlaced), Object 2 will contain Field 1 
	//! (odd field) data, and Object 3 will contain Field 2 (even field) data.
	VideoField				m_VideoData;			// 0 to 12M bytes (Depends on the object)

public:
	FcavPacketVideoData();							// 0 to 12M bytes (Depends on the object)
};

class FcavPacketData : public SpvPacket
{
protected:
	FcavPacketAncillaryData	m_AncillaryObj0;		// 4 to 259 Words

	FcavPacketAudioData		m_AudioObj1;			// 0 to 1 Word

	FcavPacketVideoData		m_VideoObj2;			// 480 bytes to 12M bytes

	FcavPacketVideoData		m_VideoObj3;			// 0 to 12M bytes

public:
	FcavPacketData();
};

/////////////////////////////////////////////////////////////////////////////
//////			         FC-AV Data - Generator					      ///////
/////////////////////////////////////////////////////////////////////////////
class FcavPacketColorPaletteGenerator : public SpvPacketGen <FcavPacketColorPalette>
{
protected:
	ExtPacketData						*m_PacketData;

public:
	FcavPacketColorPaletteGenerator(ExtPacketData *packetData, int colorPaletteNum);

	virtual void InitKeepConstrains();
};

class FcavPacketAncillaryDataGenerator : public SpvPacketGen <FcavPacketAncillaryData>
{
protected:
	int									m_ObjectNum;
	ExtPacketData						*m_PacketData;
	FcavPacketColorPaletteGenerator		**m_ColorPaletteGenerator;

public:
	FcavPacketAncillaryDataGenerator(ExtPacketData *packetData, int objectNum);

	virtual void InitKeepConstrains();
};

class FcavPacketAudioDataGenerator : public SpvPacketGen <FcavPacketAudioData>
{
protected:
	SpvUserData							m_UsrData;
	int									m_ObjectNum;
	ExtPacketData						*m_PacketData;

public:
	FcavPacketAudioDataGenerator(ExtPacketData *packetData, int objectNum);

	virtual void InitKeepConstrains();
};

class FcavPacketVideoDataGenerator : public SpvPacketGen <FcavPacketVideoData>
{
protected:
	SpvUserData							m_UsrData;
	int									m_ObjectNum;
	ExtPacketData						*m_PacketData;

public:
	FcavPacketVideoDataGenerator(ExtPacketData *packetData, int objectNum);

	virtual void InitKeepConstrains();
};

class FcavPacketDataGenerator : public SpvPacketGen <FcavPacketData>
{
protected:
	ExtPacketData						*m_PacketData;
	FcavPacketAncillaryDataGenerator	m_AncillaryGeneratorObj0;
	FcavPacketAudioDataGenerator		m_AudioGeneratorObj1;	
	FcavPacketVideoDataGenerator		m_VideoGeneratorObj2;	
	FcavPacketVideoDataGenerator		m_VideoGeneratorObj3;	

public:
	FcavPacketDataGenerator(ExtPacketData *packetData);

	virtual void InitKeepConstrains();
};

#endif