#if !defined(FCAV_PACKET_DATA_HDR_)
#define FCAV_PACKET_DATA_HDR_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Common/CommonPacketStruct.h"
#include "FcavCommon.h"

/////////////////////////////////////////////////////////////////////////////
//////			         FC-AV Data Header - Structure				  ///////
/////////////////////////////////////////////////////////////////////////////
class FcavPacketObjectN : public SpvPacket
{
protected:
	//! Includes 5 major classifications of types: Null, Assigned, Reserved, Negotiated, 
	//! and Vendor Specific.  We will typically set this to Null except for video objects in which
	//! We will set to D000h.
	SpvField				m_ObjectIndex;			// 16 Bits

	//! An unsigned byte used to link an object with another object within the
	//! Same Container.  The value is the Object Number of the associated object.
	SpvField				m_ObjectLinkPointer;	// 8 Bits

	//! Designates the type of data (video - uncompressed, video  compressed, 
	//! Video program, graphics, audio  uncompressed, audio  compressed, ancillary data, 
	//! Full stream structures, or vendor specific) To be contained with in the Object class
	SpvField				m_ObjectType;			// 8 Bits

	//! Unsigned 32-bit integer depicting the Object Size in bytes. 
	m_ObjectSizeField		m_ObjectSize;			// 1 Word

	//! Unsigned 32-bit integer which gives the offset in bytes from the beginning of the Container.
	ObjectOffsetField		m_ObjectOffset;			// 1 Word

	//! A 32-bit field which contains a definition for Vendor Specific Classifications.  If used
	//! Bit 31 is set to one, bits 24 through 30 may be defined by the vendor, bits 0 through 23 
	//! Are the IEEE OUI Company ID.
	SpvField				m_ObjectTypeDef;		// 1 Word


public:
	FcavPacketObjectN();				// 4 Words
};

class FcavPacketDataHdr : public SpvPacket
{
protected:
	FcavPacketObjectN		m_Object0;				// 4 Words
	FcavPacketObjectN		m_Object1;				// 4 Words
	FcavPacketObjectN		m_Object2;				// 4 Words
	FcavPacketObjectN		m_Object3;				// 4 Words

public:
	FcavPacketDataHdr();							// 16 Words
};


/////////////////////////////////////////////////////////////////////////////
//////			         FC-AV Data Header - Generator			      ///////
/////////////////////////////////////////////////////////////////////////////
class FcavPacketObjectNGenerator : public SpvPacketGen <FcavPacketObjectN>
{
protected:
	SpvUserData							m_UsrData;
	ExtPacketData						*m_PacketData;
	int									m_ObjectNum;

public:
	FcavPacketObjectNGenerator(ExtPacketData *packetData, int objectNum);

	virtual void InitKeepConstrains();
};

class FcavPacketDataHdrGenerator : public SpvPacketGen <FcavPacketDataHdr>
{
protected:
	ExtPacketData						*m_PacketData;
	FcavPacketObjectNGenerator			m_Object0Generator;	
	FcavPacketObjectNGenerator			m_Object1Generator;		
	FcavPacketObjectNGenerator			m_Object2Generator;		
	FcavPacketObjectNGenerator			m_Object3Generator;		

public:
	FcavPacketDataHdrGenerator(ExtPacketData *packetData);

	virtual void InitKeepConstrains();
};

#endif