// FcavPacketCol.cpp: implementation of the FcavPacketCol class.
//
//////////////////////////////////////////////////////////////////////

#include "FcavPacketCol.h"
#include <SpvPlugin.h>
#include "../Common/CommonDefs.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FcavPacketCol::FcavPacketCol()
{
}

FcavPacketCol::~FcavPacketCol()
{

}

void FcavPacketCol::Start()
{
	bool successGet = true;
	// Signal Definition
	successGet = successGet && SD::GetSig(m_SigClock,		"S_CLOCK");
	successGet = successGet && SD::GetSig(m_SigReset,		"S_RST");
	successGet = successGet && SD::GetSig(m_SigDataIn,		"S_DATA_IN");
	successGet = successGet && SD::GetSig(m_SigDataInValid,	"S_DATA_IN_VALID");
	successGet = successGet && SD::GetSig(m_SigDataOut,		"S_DATA_OUT");
	successGet = successGet && SD::GetSig(m_SigDataOutValid,"S_DATA_OUT_VALID");
	successGet = successGet && SD::GetSig(m_SigFramePulse,	"S_FRAME_PULSE_OUT");
	
	successGet = successGet && SD::GetBool(m_PrintGoodData,	"B_PRINT_GOOD_COMPARE");

	if(successGet == false) return;

	m_PosClock.Init(m_SigClock, AtPos);
	m_NegClock.Init(m_SigClock, AtNeg);
	m_PosReset.Init(m_SigReset, AtPos);
	m_NegReset.Init(m_SigReset, AtNeg);

	m_FcavFrameCnt		= 0;
	m_ImageCnt			= 0;
	m_FcavPacketUsr		= new FcavPacketStruct();
	m_FcavPacketGenUsr	= new FcavPacketGenerator(&m_PacketDataUsr);

	StartTProc(this, (SPVPM)&FcavPacketCol::PacketRestartCompare);
	m_ThreadCtrls.push_back(StartTProc(this, (SPVPM)&FcavPacketCol::PacketInThread));
	m_ThreadCtrls.push_back(StartTProc(this, (SPVPM)&FcavPacketCol::PacketOutThread));

}

void FcavPacketCol::PacketInThread()
{
	Wait(m_PosReset);

	// Starting to insert data to the fifo
	for(;;) {
		if(m_SigDataInValid.Uint() == 1)
			m_Fifo.push_back(m_SigDataIn.Uint());
		Wait(m_NegClock);
	}
}

void FcavPacketCol::PacketOutThread()
{
	unsigned dataCnt = 0;
	
	Wait(m_PosReset);

	// Starting to compare data from the fifo
	for(;;) {
		if(m_SigDataOutValid.Uint() == 1) {
			if(m_Fifo.size() == 0) {
				SPV_ERROR("FcavPacketCol::PacketOutThread - Compare failed because data in fifo is empty");
			}
			else {
                unsigned expected = TransferFunc(m_Fifo.front());
				if(!Compare(expected, m_SigDataOut.Uint())) 
					SPV_ERROR("FcavPacketCol::PacketOutThread - Data["+Itos(dataCnt)+"] failed Received="+Itos(m_SigDataOut.Uint())+" Different Expected="+Itos(expected));
				else 
					SPV_DEBUG("FcavPacketCol::PacketOutThread - Compare OK, Sample #"+Itos(dataCnt)+", Received="+Itos(m_SigDataOut.Uint()),DBG_LVL_TB);

				BuildFcavPacket();
				
				m_Fifo.pop_front();
			}
			dataCnt++;
		}
		Wait(m_PosClock);
	}
}

unsigned FcavPacketCol::TransferFunc(unsigned input)
{
    return input;
}

bool FcavPacketCol::Compare(unsigned expected, unsigned received)
{
    return expected == received;
}

void FcavPacketCol::PacketRestartCompare()
{
	for(;;) {
		Wait(m_NegReset);
        m_Fifo.clear();

		for(int i = 0; i < m_ThreadCtrls.size(); i++){
			m_ThreadCtrls[i]->Restart();
		}
	}
}

void FcavPacketCol::BuildFcavPacket()
{
	if(m_SigFramePulse.Uint() == 1)
	{ 
		if(m_FcavFrameCnt > 0)
		{
			SPV_OUT(<<"\n\n*** Packet #"<<m_FcavFrameCnt<<" Collected ***\n"<<(*m_FcavPacketUsr)<<endl<<"Time Is: " << SpvSig::time<<endl<<endl);

			if(m_PacketDataUsr.m_FramesFieldsGen.GetLastVal() == SD::GetInt("N_FCAV_FRAMES_INTERLACED"))
			{
				bool retVal;
				m_OddImage.SetVector(m_FcavPacketUsr->FindField("VideoData2",retVal));
				m_OddImage.CreateFile("OddImage.bmp");
				m_EvenImage.SetVector(m_FcavPacketUsr->FindField("VideoData3",retVal));
				m_EvenImage.CreateFile("EvenImage.bmp");
				   
				if(m_FullImage.SetInterlacedImages(m_OddImage,m_EvenImage))
					system(m_FullImage.CreateFile("FullInterlaced.bmp").c_str());
			}
			else  
			{
				bool retVal;
				m_FullImage.SetVector(m_FcavPacketUsr->FindField("VideoData2",retVal));
				system(m_FullImage.CreateFile("FullProgressive.bmp").c_str());
			}

		}

		// Setting the FCAV Frame
		if(!SD::IsExists("TEST_IMAGE_FILE",StrType,m_ImageCnt))
		{
			if(m_ImageCnt == 0) return;
			m_ImageCnt = 0;
		}

		if(!m_PacketDataUsr.m_Video.OpenFile(SD::GetStr("TEST_IMAGE_FILE",m_ImageCnt), ImageFormat(SD::GetInt("TEST_IMAGE_TYPE",m_ImageCnt)))) return;

		m_OddImage.CreateEmptyImage(m_PacketDataUsr.m_Video.GetWidth(), m_PacketDataUsr.m_Video.GetInterlacedOddHeight(), m_PacketDataUsr.m_Video.GetFormat(), m_PacketDataUsr.m_Video.GetBpp(), m_PacketDataUsr.m_Video.GetColorDepth(), m_PacketDataUsr.m_Video.GetColorType());
		m_EvenImage.CreateEmptyImage(m_PacketDataUsr.m_Video.GetWidth(), m_PacketDataUsr.m_Video.GetInterlacedEvenHeight(), m_PacketDataUsr.m_Video.GetFormat(), m_PacketDataUsr.m_Video.GetBpp(), m_PacketDataUsr.m_Video.GetColorDepth(), m_PacketDataUsr.m_Video.GetColorType());
		m_FullImage.CreateEmptyImage(m_PacketDataUsr.m_Video.GetWidth(), m_PacketDataUsr.m_Video.GetHeight(), m_PacketDataUsr.m_Video.GetFormat(), m_PacketDataUsr.m_Video.GetBpp(), m_PacketDataUsr.m_Video.GetColorDepth(), m_PacketDataUsr.m_Video.GetColorType());
		 	    
		SetDeafultExtPacketData(&m_PacketDataUsr);
		m_PacketDataUsr.m_NumColumnsGen.SetVal(m_PacketDataUsr.m_Video.GetWidth());
		m_PacketDataUsr.m_NumRowsGen.SetVal(m_PacketDataUsr.m_Video.GetHeight());
		m_PacketDataUsr.m_ObjectNSizeGen[1].SetVal(0);
		m_FcavPacketUsr = m_FcavPacketGenUsr->GenPacket();

		m_FcavPacketUsr->StartCollection();
		m_FcavFrameCnt++;
	}

	m_FcavPacketUsr->CollectData(m_SigDataOut.Uint(),m_SigDataOut.Size());
}
