// FcavPacketDriver.cpp: implementation of the FcavPacketDriver class.
//
//////////////////////////////////////////////////////////////////////

#include "FcavPacketDriver.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FcavPacketDriver::FcavPacketDriver()
{

}

FcavPacketDriver::~FcavPacketDriver()
{

}

void FcavPacketDriver::Start()
{	
	bool successGet = true;

	// Signal Definition
	successGet = successGet && SD::GetSig(m_SigClock,		"S_CLOCK");
	successGet = successGet && SD::GetSig(m_SigReset,		"S_RST");
	successGet = successGet && SD::GetSig(m_SigDataIn,		"S_DATA_IN");
	successGet = successGet && SD::GetSig(m_SigDataInValid,	"S_DATA_IN_VALID");
	successGet = successGet && SD::GetSig(m_SigFramePulse,	"S_FRAME_PULSE_IN");

	// Generators Definition
	successGet = successGet && SD::GetGen(m_DataEnGen,		"G_DATA_EN");

	if(successGet == false) return;

	m_ImageCnt			= 0;
	m_FcavFrameCnt		= 0;
	m_PacketPos			= 0;
	m_FcavPacketUsr		= new FcavPacketStruct();
	m_FcavPacketGenUsr	= new FcavPacketGenerator(&m_PacketDataUsr);

	SpvPacket::UpdateInGen(); 

	// Init the events
	m_PosClock.Init(m_SigClock, AtPos);
	m_PosReset.Init(m_SigReset, AtPos);
	m_NegReset.Init(m_SigReset, AtNeg);

	StartTProc(this, (SPVPM)&FcavPacketDriver::ThreadReset);
	StartTProc(this, (SPVPM)&FcavPacketDriver::ThreadRestartData);
	m_ThreadCtrls.push_back(StartTProc(this, (SPVPM)&FcavPacketDriver::ThreadDriveData));
}

void FcavPacketDriver::ThreadDriveData() 
{
	m_PixVal.Resize(m_SigDataIn.Size());

	Wait(1);

	m_SigDataInValid	= 0;
	m_SigDataIn			= 0;
	m_SigFramePulse		= 0;

	Wait(m_PosReset);
	m_PacketPos = m_PacketVec.Size();  

	// This function will drive Generated data in to the DUT
	for(;;) {
		unsigned dataEnCnt = m_DataEnGen->Gen();

		// This while loop will Generate the amount of clocks that valid should be up
		while(dataEnCnt-- > 0) {
			m_SigDataInValid	= 1;
			if(!Gen()) return;
			Wait(m_PosClock);
		}

		m_SigDataInValid	= 0;
		Wait(m_PosClock);
	}

	Wait(m_PosClock);
}

bool FcavPacketDriver::Gen()
{
	m_SigFramePulse = 0;
	
	for(int i = 0; i < m_PixVal.Size(); i++)
	{
		if(m_PacketPos == m_PacketVec.Size())
		{
			if(!MakePacket(m_PacketVec)) 
			{
				SPV_ERROR("FcavPacketDriver::Gen - MakePacket Failed");
				return false;
			}
			m_SigFramePulse = 1;
			m_PacketPos = 0;
		}

		m_PixVal[i] = m_PacketVec[m_PacketPos++].Uint() != 0;
	}

	m_SigDataIn = m_PixVal.Uint();
	return true;
}

bool FcavPacketDriver::MakePacket(SpvBitVec &currPacketVec)
{
	// Setting the FCAV Frame
	if(!SD::IsExists("TEST_IMAGE_FILE",StrType,m_ImageCnt))
	{
		if(m_ImageCnt == 0)
			return false;
		m_ImageCnt = 0;
	}

	if(!m_PacketDataUsr.m_Video.OpenFile(SD::GetStr("TEST_IMAGE_FILE",m_ImageCnt), ImageFormat(SD::GetInt("TEST_IMAGE_TYPE",m_ImageCnt))))
		return false;
	
	SetDeafultExtPacketData(&m_PacketDataUsr);
	m_PacketDataUsr.m_NumColumnsGen.SetVal(m_PacketDataUsr.m_Video.GetWidth());
	m_PacketDataUsr.m_NumRowsGen.SetVal(m_PacketDataUsr.m_Video.GetHeight());
   
	if(m_PacketDataUsr.m_Video.GetColorType() == GrayScale)
		m_PacketDataUsr.m_ColorInfoTypeGen.SetVal(SD::GetInt("N_FCAV_COLOR_MONOCHROME"));
	else if(m_PacketDataUsr.m_Video.GetColorType() == RGBClr)
		m_PacketDataUsr.m_ColorInfoTypeGen.SetVal(SD::GetInt("N_FCAV_COLOR_RGB"));
	else if(m_PacketDataUsr.m_Video.GetColorType() == RGBAClr)
		m_PacketDataUsr.m_ColorInfoTypeGen.SetVal(SD::GetInt("N_FCAV_COLOR_RGBA"));

	m_PacketDataUsr.m_ObjectNSizeGen[1].SetVal(0);

	m_FcavPacketUsr = m_FcavPacketGenUsr->GenPacket();

	m_FcavFrameCnt++;
    SPV_OUT(<<"\n\n*** Packet #"<<m_FcavFrameCnt<<" Sent ***\n"<<(*m_FcavPacketUsr)<<endl<<"Time Is: " << SpvSig::time<<endl<<endl);

	currPacketVec.Copy(m_FcavPacketUsr->PackAll());
	
	m_ImageCnt++;

	return true;
}

void FcavPacketDriver::ThreadReset()
{
	// Waiting 2 Clocks, Reseting for 2 clocks and continue
	Wait(m_PosClock,2);
	m_SigReset	= 0;
	Wait(m_PosClock,2);
	m_SigReset	= 1;
}
	

// Restart function that listen for a reset event	
void FcavPacketDriver::ThreadRestartData()
{
	for(;;) {
		Wait(m_NegReset);
		for(int i = 0; i < m_ThreadCtrls.size(); i++){
			m_ThreadCtrls[i]->Restart();
		}
	}
}

