#ifndef PacketStruct
#define PacketStruct
#include "spvPacketAndTraffic.h"
#include "SpvPlugIn.h"

///////////////////////////////////////////////////////////////////////
//////////////////   NestedA Packet and Generator  ////////////////////
///////////////////////////////////////////////////////////////////////
class	NestedA	: public SpvPacket
{
public:
	SpvField	mFieldA1;
	SpvField	mFieldA2;
	SpvField	mFieldA3;
	NestedA() {
		mFieldA1.MakePhysical(this);//32 bits.
		mFieldA2.MakePhysical(this,12);
		mFieldA3.MakePhysical(this,12);
	}
};

class	NestedAGenerator : public SpvPacketGen<NestedA>
{
public:
	NestedAGenerator() {
		InitKeepConstrains();
	};
	void				InitKeepConstrains() {
		KeepRange(mFieldA1,"5 20,100 200,300","Range");
		KeepConst(mFieldA2,150,"Const");
		KeepRelation(mFieldA3,SmallerThan,mFieldA1,"Relation");

	}
};

///////////////////////////////////////////////////////////////////////
////////////////    End NestedA Packet and Generator  /////////////////
///////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////
//////////////////   NestedA Packet and Generator  ////////////////////
///////////////////////////////////////////////////////////////////////
class	NestedB	: public	SpvPacket
{
public:
	SpvField	mFieldB1;
	SpvField	mFieldB2;
	SpvField	mFieldB3;
	SpvField	mFieldB4;
	NestedB() {
		mFieldB1.MakePhysical(this,12);//32 bits.
        mFieldB2.MakePhysical(this,SD::GetSig("SIG1")->Size());
		mFieldB3.MakePhysical(this,SD::GetSig("SIG1")->Size());
		mFieldB4.MakePhysical(this,SD::GetSig("SIG1")->Size());
	}
};

class	NestedBGenerator : public SpvPacketGen<NestedB>
{
	CrossCoverGeneration			MCoverGen;
public:
	NestedBGenerator() {
		MCoverGen.Init(SD::GetCov("COVERSIG12"));
		InitKeepConstrains();
	};

	virtual void		InitKeepConstrains() {
        KeepGeneration(mFieldB1,SD::GetGen("GENSTEP"),"Step");
		KeepCover(mFieldB2,SD::GetCov("COVERSIG1"),"SingleCover");
		KeepCrossCover(mFieldB3,&MCoverGen,0,"CrossItem0");
		KeepCrossCover(mFieldB4,&MCoverGen,1,"CrossItem1");
	}
};

///////////////////////////////////////////////////////////////////////
//////////////////   End NestedB Packet and Generator  ////////////////////
///////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////
//////////////////   Global Packet and Generator  /////////////////////
///////////////////////////////////////////////////////////////////////

class	Num1Field : public SpvField
{
	static	unsigned mDataToMakeSum1;
public:
	void	initDataSource(SpvField &dataRef) {mDataToMakeSum1 = dataRef.Ind();};
	SpvBitVec	&RefVal() {return Father()->GetField(mDataToMakeSum1);};
	virtual	SpvField&	Generate(SpvUserData *Data=NULL) {
		MeVec() = RefVal().Sum1();
		if (!IsPhysical()) {
			SetNotPhysical();
		}
		return *this;
	}
	virtual	bool	IsPhysical(){
		if (RefVal().Uint() <= 400) {
			SetNotPhysical();
			return false;
		} else {
			return true;
		}
	}
};



class	TimeStamp : public SpvField
{
	virtual	SpvField&	Generate(SpvUserData *Data=NULL) {
		MeVec() = SpvSig::time;
		return *this;
	}

};


class	MyGlobalPacketStruct	:	public SpvPacket
{
public:
	SpvField		mPortIdField;
	SpvField		mChannelIdField;
	SpvField		mClassServiceField;
	NestedA			mFieldPacketNestedA;
	SpvField		mLengthField;
	spvPayLoadField	mPayLoadField;
	SpvField		mCrcDataField;
	NestedB			mFieldPacketNestedB;
	SpvField		mPreiousParityField;
	SpvField		mFieldParityField;
	Num1Field		mNum1Field;
	TimeStamp		mTime;
	MyGlobalPacketStruct() {
		mPortIdField.MakePhysical(this,8);//there are 100 ports
		mChannelIdField.MakePhysical(this,4);//there are 10 ports
		mClassServiceField.MakePhysical(this,4);//there are 7 kind of class service
		mFieldPacketNestedA.MakePhysical(this);
		mLengthField.MakePhysical(this,8);//Size is in bytes
		mPayLoadField.MakePhysical(this);//Size is Dynamic
		mCrcDataField.MakePhysical(this,16);
		mFieldPacketNestedB.MakePhysical(this);
		mPreiousParityField.MakePhysical(this,4);
		mFieldParityField.MakePhysical(this,4);
		mNum1Field.MakePhysical(this,20);
		mTime.MakePhysical(this,64);
		mNum1Field.initDataSource(mPayLoadField);
	}

	virtual		void			PreGeneration() {
		mPayLoadField.SetBitSizeFactor(8);
	};

	virtual		void			PreGetEmpty()   {//
		mPayLoadField.SetBitSizeFactor(8);
	}

	virtual		void			PostGeneration() {};
};

class	GlobPacketGenerator: public SpvPacketGen<MyGlobalPacketStruct>
{
	unsigned			mPortId;
	unsigned			mChannelId;
	NestedAGenerator	mNestedAGenerator;
	NestedBGenerator	mNestedBGenerator;
public:
	GlobPacketGenerator() {
		//InitKeepConstrains();
	};
	void	InitPortChannel(unsigned PortId,unsigned ChannelId) {mPortId = PortId;mChannelId = ChannelId;};
	virtual		void	InitKeepConstrains() {
		KeepConst(mPortIdField,mPortId,"PortId");

		KeepConst(mChannelIdField,mChannelId,"ChannelId");

		KeepGeneration(mClassServiceField,SD::GetGen("GEN_CLASS_SERVICE",mPortId),"Class");

		KeepNestedPacket(mFieldPacketNestedA,&mNestedAGenerator,"NestedA");

		KeepGeneration(mLengthField,SD::GetGen("GEN_LENGTH",mPortId),"Length");

		//mPayLoadField  generated automatically by virtual function
		KeepCrcData(mCrcDataField,mPayLoadField,16,0x8005,0,0,true,"Crc");

		KeepNestedPacket(mFieldPacketNestedB,&mNestedBGenerator,"NestedB");

		KeepAllPreviousParity(mPreiousParityField,"AllParity");

		KeepFieldParity(mFieldParityField,mPayLoadField,"PayLoadParity");
		//mNum1Field generated automatically by virtual function
		SetFieldName(mNum1Field,"NUM1");

	//	SetFieldName(
	}
};
#endif
